/*
 *  TOPPERS/MDCOM Multi Domain Communication Module
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Multi Domain Communication Module
 *
 *  Copyright (C) 2012-2019 by Embedded and Real-Time Systems Laboratory
 *     Graduate School of Information Science, Nagoya University, JAPAN
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: mdcom_kernel.h 860 2016-09-18 07:58:22Z ertl-honda $
 */

#ifndef MDCOM_KERNEL_H
#define MDCOM_KERNEL_H

#include <Os.h>
#include <t_syslog.h>

#define COPY_UPDATE_ALGORITHM_ENABLE

#define DEBUG(enable,x,args...) \
			if(enable) syslog(LOG_NOTICE, "DEBUG (%s): " x, __func__ , ##args);

#define PRINT(x, args...) \
			syslog(LOG_NOTICE, x, ##args);

#define	assert(x)	if( !(x) )	{	\
						syslog(LOG_NOTICE, "ASSERT : %s  line %d\n", __FILE__, __LINE__);	\
						if( TerminateTask() != E_OK )  while(1);	\
					};

#define	assert_isr(x)	if( !(x) )	{	\
						syslog(LOG_NOTICE, "ASSERT : %s  line %d\n", __FILE__, __LINE__);	\
						return;	\
					};

/*
 * Type definition
 */
typedef uint32_t				size_t;
#ifndef NULL
#define NULL		0			/* 無効ポインタ */
#endif /* NULL */

/*
 * TASK EXC ARG
 */
#define TASK_EXC_ARG_PROCESSEXIT 1
#define TASK_EXC_ARG_TERM        2

/*
 * Driver State
 */
#define MDCOM_KERNEL_NOINIT     0 /* __mdcom_kernel_init() is not called */
#define MDCOM_KERNEL_REQINIT    1 /* __mdcom_kernel_init() send CTL_M2R_REQ_INIT and wait responses from NT_OS */
#define MDCOM_KERNEL_INITD      2 /* Recieved CTL_R2M_REQ_INIT from NT_OS and initialization is done */

/*
 * KERNEL INTERFACE
 */
extern int __mdcom_kernel_init(mdcom_mask_t fifoch_mask, mdcom_time_t timeout, void *sregion);
extern int __mdcom_kernel_term(void);
extern int __mdcom_kernel_get_state(void);
extern int __mdcom_kernel_sregion_init(void **sregion);
extern int __mdcom_kernel_notify(mdcom_id_t fifoch_id);
extern int __mdcom_kernel_wait(mdcom_id_t fifoch_id, mdcom_time_t timeout);
extern int __mdcom_kernel_get_fifoch_mask(mdcom_mask_t *p_fifoch_mask);
extern int __mdcom_kernel_fifoch_init(const mdcom_id_t fifoch_id);
extern int __mdcom_kernel_get_num_process(uint32_t *p_num_process);
extern void __mdcom_kernel_lock(void);
extern void __mdcom_kernel_unlock(void);

extern void   __mdcom_kernel_mutex_init(void);
extern bool_t __mdcom_kernel_mutex_trylock(uint32_t *p_lock);
extern void   __mdcom_kernel_mutex_unlock(uint32_t  *p_lock);

typedef struct {
	TaskType		rtos_tskid;
	EventMaskType	rtos_event;
	uint32_t		timeout_time;
	uint32_t		IsTimeouted;
} mdcom_kernel_wait_t;

extern int __mdcom_kernel_set_timeout(mdcom_kernel_wait_t *wait_request);
extern int __mdcom_kernel_clear_timeout(mdcom_kernel_wait_t *wait_request);

static inline bool_t
__mdcom_kernel_check_tskctx()
{
	StatusType	ercd;

	ercd = ClearEvent(0);
	if( ercd == E_OK )  {
		return true;
	}
	else  {
		return false;
	}
}

/*
 * ISR
 */
extern void mdcom_handler(intptr_t exinf);

#endif /* MDCOM_KERNEL_H */
