/*
 *  @(#) $Id: mdcom_kernel.h 849 2016-07-19 12:42:07Z ertl-honda $
 */
#ifndef MDCOM_KERNEL_H
#define MDCOM_KERNEL_H

#include <unistd.h>    /* ftruncate */
#include <fcntl.h>     /* flags: O_CREAT, etc */
#include <sys/stat.h>  /* modes: S_IRWXU, etc */
#include <sys/mman.h>  /* shm_xxx and mmap() */
#include <semaphore.h> /* sem_xxx */
#include <assert.h>    /* assert() */
#include <string.h>    /* memset */
#include <pthread.h>   /* pthread_xxx */
#include <sys/ioctl.h>
#include <stdio.h>

#define DEBUG(enable,x,args...) \
			if(enable) printf("DEBUG (%s): " x, __func__ , ##args)

#define PRINT(x, args...) \
			printf("\n"); printf(x, ##args);

/*
 * KERNEL INTERFACE
 */
extern int __mdcom_kernel_init(mdcom_id_t fifoch_id, mdcom_time_t timeout, void *sregion);
extern int __mdcom_kernel_term(void);
extern int __mdcom_kernel_sregion_init(void **sregion);
extern int __mdcom_kernel_notify(mdcom_id_t fifoch_id);
extern int __mdcom_kernel_wait(mdcom_id_t fifoch_id, mdcom_time_t timeout);
extern int __mdcom_kernel_get_fifoch_mask(mdcom_mask_t *p_fifoch_mask);
extern int __mdcom_kernel_fifoch_init(const mdcom_id_t fifoch_id);
extern int __mdcom_kernel_get_num_process(uint32_t *p_numb_process);

extern void   __mdcom_kernel_mutex_init(void);
extern bool_t __mdcom_kernel_mutex_trylock(uint32_t *p_lock);
extern void   __mdcom_kernel_mutex_unlock(uint32_t  *p_lock);

static inline bool_t
__mdcom_kernel_check_tskctx()
{
	return true;
}

extern pthread_mutex_t mutex;

static inline void
__mdcom_kernel_lock()
{
	pthread_mutex_lock(&mutex);
}

static inline void
__mdcom_kernel_unlock()
{
	pthread_mutex_unlock(&mutex);
}
#endif /* MDCOM_KERNEL_H */
