/*
 *  TOPPERS/MDCOM
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Multi Domain Communication Module
 *
 *  Copyright (C) 2012-2015 by Embedded and Real-Time Systems Laboratory
 *     Graduate School of Information Science, Nagoya University, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: mdcom.h 1034 2019-03-07 16:25:46Z ertl-honda $
 */

#ifndef MDCOM_H
#define MDCOM_H

/*
 *  MDCOM Global Definition
 */

/*
 * MDCOM_TYPE_API
 */
#define MDCOM_TYPE_API_POSIX    (1)
#define MDCOM_TYPE_API_TOPPERS  (2)

/*
 * MDCOM_TYPE_OS
 */
#define MDCOM_TYPE_OS_LINUX     (1)
#define MDCOM_TYPE_OS_ASP       (2)
#define MDCOM_TYPE_OS_FMP       (3)

/*
 * MDCOM_TYPE_EMV
 */
#define MDCOM_TYPE_EMV_AMP      (1)      
#define MDCOM_TYPE_EMV_SAFEG    (2)

/*
 * MDCOM_TYPE_ROLL
 */
#define MDCOM_TYPE_ROLL_MASTER  (1)
#define MDCOM_TYPE_ROLL_REMOTE  (2)
    
/*
 * Target Header
 */
#include "mdcom_target.h"

#if MDCOM_TYPE_API == MDCOM_TYPE_API_POSIX
typedef bool bool_t;
#endif /* MDCOM_TYPE_API == MDCOM_TYPE_API_POSIX */

/*
 * ERRORS
 */
#define MDCOM_SUCCESS    (0)  // success
#define MDCOM_NOINIT    (-1)  // not initialized yet.
#define MDCOM_EPARAM    (-2)  // incorrect parameter.
#define MDCOM_FILTER    (-3)  // the block was discarded
#define MDCOM_EALLOC    (-4)  // the block is not allocated.
#define MDCOM_TIMEOUT   (-5)  // a timeout occurred
#define MDCOM_LOCKED    (-6)  // the shmem is locked
#define MDCOM_UNLOCKED  (-7)  // the shmem is unlocked
#define MDCOM_ESREGION  (-8)  // the shared regison initialize error
#define MDCOM_RESTART   (-9)  // restart fifoch
#define MDCOM_ECTX      (-10) // context error
#define MDCOM_EASSIGN   (-11) // the fifo channel is assigned other process
#define MDCOM_EMPTY     (-12) // no block is enqueued.
#define MDCOM_FULL      (-13) // no block is in buffer.
#define MDCOM_WAITOVER  (-14) // an other task is waitting for the channel
#define MDCOM_OPROCESS  (-15) // number of connected process is over
#define MDCOM_EINIT     (-16) // NT_OS calls init before T_OS calls init
#define MDCOM_EOBJ      (-17) // Object state error.
#define MDCOM_OTHER     (-18) // Other error


/*
 * TYPES
 */
typedef int32_t mdcom_id_t;
typedef int32_t mdcom_time_t;
typedef bool_t (*mdcom_filter_t)(void *buffer, uint32_t size);
typedef int32_t mdcom_mask_t;

/*
 * MACRO
 */
#define NULL_FILTER			(-1)
#define DEFAULT_FILTER		(-2)


/*
 *  API for Global
 */

/*
 * mdcom_init
 *
 * Initializates the Dual-OS Communications system.
 *
 * Errors: MDCOM_EASSIGN, MDCOM_TIMEOUT, MDCOM_SHREGION, MDCOM_ECTX,
 *         MDCOM_EPARAM, MDCOM_OPROCESS
 *
 */
extern int mdcom_init(mdcom_mask_t fifoch_mask, mdcom_time_t timeout);

/*
 * mdcom_term
 *
 * Terminate the Dual-OS Communications system.
 *
 * Errors: MDCOM_NOINIT
 *
 */
extern int mdcom_term(void);

/*
 * mdcom_sense_process
 *
 * Sense number of process(in T_OS) that open /dev/safeg.
 *
 * Errors: MDCOM_NOINIT
 */
extern int mdcom_sense_process(uint32_t *p_num_process);

/*
 * mdcom_sense_fifoch
 *
 * Sense FIFO Channel status
 *
 * Errors: MDCOM_NOINIT  
 */
extern int mdcom_sense_fifoch(mdcom_mask_t *p_fifoch_mask);

/*
 *  API for FIFO CHANNEL
 */

/*
 * mdcom_fifoch_alloc
 *
 * Allocates a block from a fifo channel block pool and get a pointer to
 * the beginning of the memory region of the allocated block.
 * This function never blocks the calling task or process.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN, MDCOM_EALLOC
 *
 */
extern int mdcom_fifoch_alloc(const mdcom_id_t fifoch_id,
								mdcom_id_t *p_block_id);

/*
 * mdcom_fifoch_free
 *
 * Releases a block back to the fifo channel block pool where it belongs.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN, MDCOM_EALLOC
 *
 */
extern int mdcom_fifoch_free(const mdcom_id_t fifoch_id,
								const mdcom_id_t block_id);

/*
 * mdcom_fifoch_get
 *
 * To obtain a pointer to the beginning of the memory region of a block.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EALLOC
 *
 */
extern int mdcom_fifoch_get(const mdcom_id_t fifoch_id,
								const mdcom_id_t block_id, void **p_block);

/*
 * mdcom_fifoch_enqueue
 *
 * Enqueues a block to a fifo channel FIFO. Note that the fifo channel is implicit in
 * the block identifier.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN, MDCOM_FILTER, MDCOM_EALLOC
 *
 */
extern int mdcom_fifoch_enqueue(const mdcom_id_t fifoch_id,
									const mdcom_id_t block_id);

/*
 * mdcom_fifoch_dequeue
 *
 * Dequeues a block from a fifo channel FIFO. This function never blocks the
 * calling task or process.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN, MDCOM_EALLOC
 *
 */
extern int mdcom_fifoch_dequeue(const mdcom_id_t fifoch_id,
									mdcom_id_t *p_block_id);

/*
 * mdcom_fifoch_notify
 *
 * Sends an asynchronous fifo channel event notification. If a notification
 * was already sent and has not been acknowledged yet by the receiver, the
 * function will return as if the notification was sent successfully.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN
 *
 */
extern int mdcom_fifoch_notify(const mdcom_id_t fifoch_id);

/*
 * mdcom_fifoch_wait
 *
 * This function makes the calling task - or process - wait for an
 * asynchronous event notification on a specific fifo channel. If an event was
 * pending, the function acknowledges it and returns immediately.
 * Otherwise, the calling thread is put in waiting state until an event
 * arrives or a timeout occurs. The units in which the timeout is
 * expressed are implementation-dependent.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN, MDCOM_TIMEOUT,
 *         MDCOM_ECTX, MDCOM_RESTART
 *
 */
extern int mdcom_fifoch_wait(const mdcom_id_t fifoch_id,
								const mdcom_time_t timeout);

/*
 * mdcom_filter_set
 *
 * Used by the receiver to set a communications filter.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN
 *  
 */
extern int mdcom_filter_set(const mdcom_id_t fifoch_id,
								const mdcom_id_t  filter_id);

/*
 * mdcom_fifoch_init
 *
 * Initialize a fifoch channel.
 * 
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_EASSIGN
 *  
 */
extern int mdcom_fifoch_init(const mdcom_id_t fifoch_id);

/*
 *  API for SMEM CHANNEL
 */

/*
 * mdcom_smemch_get
 *
 * To obtain a pointer to the beginning of the memory region of a smem channel.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM
 *
 */
extern int mdcom_smemch_get(const mdcom_id_t smemch_id, void **p_smem);

/*
 * mdcom_smemch_trylock
 *
 * Try to get lock of a smem channel.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_LOCKED
 *
 */
extern int mdcom_smemch_trylock(const mdcom_id_t smemch_id);

/*
 * mdcom_smemch_unlock
 *
 * Release lock of a smem.
 *
 * Errors: MDCOM_NOINIT, MDCOM_EPARAM, MDCOM_UNLOCKED
 */
extern int mdcom_smemch_unlock(const mdcom_id_t smemch_id);

/*
 * Kernel Header
 */
#include "mdcom_kernel.h"

#endif /* MDCOM_H */
