/*
 *  TOPPERS ATK2
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel Version 2
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2019 by Center for Embedded Computing Systems
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2011-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2011-2013 by Spansion LLC, USA
 *  Copyright (C) 2011-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2011-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2011-2014 by Renesas Electronics Corporation, JAPAN
 *  Copyright (C) 2011-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2011-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2004-2017 by Witz Corporation
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id: sample1_m_atk2.c 1010 2019-02-01 02:07:13Z ertl-honda $
 */

#include "Os.h"
#include "t_syslog.h"
#include "t_stdlib.h"
#include "sysmod/serial.h"
#include "sysmod/syslog.h"
#include "sample1_m_atk2.h"

#include "sysmod/banner.h"
#include "target_sysmod.h"
#include "target_serial.h"
#include "sysmod/serial.h"
#include "target_hw_counter.h"
#include "mdcom.h"
#include "mdcom_config.h"


bool_t      app_terminate;


//#define	WAITMODE_POLLING
#define	WAITMODE_TIMEOUT

#define	TIMEOUT_TIME	100		/* msec単位 */
#define	POLLING_TIME	(10 * TICK_FOR_10MS)

/*
 *  関数プロトタイプ宣言
 */
sint32 main(void);
TASK(MainTask);
static void 	share_mem_test(void);

extern void		*memcpy(void *dest, const void *src, uint32_t len);


/*
 *  マクロ
 */
#define GetAppModeInfo()	(0)

/*
 *  ユーザメイン関数
 *
 *  アプリケーションモードの判断と，カーネル起動
 */
sint32
main(void)
{
	AppModeType	crt_app_mode;
	StatusType	ercd;
	CoreIdType	i;

	/*
	 *  アプリケーションモードの判断
	 */
	switch (GetAppModeInfo()) {
	case 0:
		crt_app_mode = AppMode1;
		break;
	case 1:
		crt_app_mode = AppMode2;
		break;
	default:
		crt_app_mode = AppMode3;
		break;
	}

	if (GetCoreID() == OS_CORE_ID_MASTER) {
		for (i = 0; i < TNUM_STARTCORE; i++) {
			if (i != OS_CORE_ID_MASTER) {
				StartCore(i, &ercd);
			}
		}
		/*
		 *  カーネル起動
		 */
		StartOS(crt_app_mode);
	}
	else {
		/*
		 *  カーネル起動
		 */
		StartOS(DONOTCARE);
	}


	while (1) {
	}
}   /* main */


/*
 *  メインタスク
 */
TASK(MainTask)
{
    mdcom_id_t	block_id;
    int			ret;
    void		*buffer;
    uint32_t	size;
    char		msg[] = "hello world";

    size = 0;

    syslog_msk_log(LOG_UPTO(LOG_DEBUG));

#if		CYC_TIME > 0
	SetRelAlarm(CycArm, (CYC_TIME / 10) * TICK_FOR_10MS, (CYC_TIME / 10) * TICK_FOR_10MS);
#endif	/* CYC_TIME > 0 */

    while( 1 )
    {
        app_terminate = false;

	    syslog(LOG_NOTICE, "MDCOM basic test program starts.");

	    syslog(LOG_NOTICE, "Master mdcom_init");

	#ifdef	WAITMODE_POLLING
		do  {
	    	ret = mdcom_init(0x3, 0);
		}
		while( ret == MDCOM_TIMEOUT );
	#elif	defined(WAITMODE_TIMEOUT)
	#if		0
	    syslog(LOG_NOTICE, "mdcom_init() started.");
		ret = mdcom_init(0x3, 10000);
	    syslog(LOG_NOTICE, "mdcom_init() ended.");
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_init() failed.  ret=%d", ret);
			TerminateTask();
		}
	#else
		do  {
	    	ret = mdcom_init(0x3, TIMEOUT_TIME);
		}
		while( ret == MDCOM_TIMEOUT );
	#endif
	#else	/* WAITMODE */
	    ret = mdcom_init(0x3, -1);
	#endif	/* WAITMODE */
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR: mdcom_init() failed.  ret=%d", ret);
			TerminateTask();
		}

	    syslog(LOG_NOTICE, "MDCOM communications initialized");

	    /* phase 1 */
	    syslog(LOG_NOTICE, "Master send message in channel 0");

	    ret = mdcom_fifoch_alloc(MDCOM_CHANNEL_0, &block_id);
	    assert(ret == 0);

	    ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	    assert(ret == 0);
	    syslog(LOG_NOTICE, "buffer 0x%x, Size %d.", buffer, size);

	    memcpy(buffer, (void *)msg, sizeof(msg));

	    ret = mdcom_fifoch_enqueue(MDCOM_CHANNEL_0, block_id);
	    assert(ret == 0);

	    syslog(LOG_NOTICE, "Master send event");
	    ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	    assert(ret == 0);

	    syslog(LOG_NOTICE, "Master wait for ack");
	#ifdef	WAITMODE_POLLING
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#elif	defined(WAITMODE_TIMEOUT)
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, TIMEOUT_TIME);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#else	/* WAITMODE */
	    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	#endif	/* WAITMODE */
        if( app_terminate == true )  {
            continue;
        }
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_fifoch_wait() failed.  ret=%d", ret);
			TerminateTask();
		}


	    /* phase 2 */
	    syslog(LOG_NOTICE, "Master send params 3 and 5");

	    *((uint32_t *)buffer) = 3;
	    *((uint32_t *)((uintptr_t)buffer + 4)) = 5;

	    ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	    assert(ret == 0);

	    syslog(LOG_NOTICE, "Master wait result");
	#ifdef	WAITMODE_POLLING
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#elif	defined(WAITMODE_TIMEOUT)
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, TIMEOUT_TIME);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#else	/* WAITMODE */
	    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	#endif	/* WAITMODE */
        if( app_terminate == true )  {
            continue;
        }
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_fifoch_wait() failed.  ret=%d", ret);
			TerminateTask();
		}

	    syslog(LOG_NOTICE, "Master rpc result: 3 x 5 = %u", *((uint32_t *)buffer));


	    /* phase 3 */
	    syslog(LOG_NOTICE, "Master send params 4 and 6");

	    *((uint32_t *)buffer) = 4;
	    *((uint32_t *)((uintptr_t)buffer + 4)) = 6;

	    ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	    assert(ret == 0);

	    syslog(LOG_NOTICE, "Master wait result");
	#ifdef	WAITMODE_POLLING
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#elif	defined(WAITMODE_TIMEOUT)
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, TIMEOUT_TIME);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#else	/* WAITMODE */
	    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	#endif	/* WAITMODE */
        if( app_terminate == true )  {
            continue;
        }
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_fifoch_wait() failed.  ret=%d", ret);
			TerminateTask();
		}

	    syslog(LOG_NOTICE, "Master rpc result: 4 x 6 = %u", *((uint32_t *)buffer));

	    ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	    assert(ret == 0);


	    /* phase 4 */
	    syslog(LOG_NOTICE, "Master wait for sentence from GPOS in channel 1");
	#ifdef	WAITMODE_POLLING
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_1, 0);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#elif	defined(WAITMODE_TIMEOUT)
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_1, TIMEOUT_TIME);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#else	/* WAITMODE */
	    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_1, -1);
	#endif	/* WAITMODE */
        if( app_terminate == true )  {
            continue;
        }
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_fifoch_wait() failed.  ret=%d", ret);
			TerminateTask();
		}

	    ret = mdcom_fifoch_dequeue(MDCOM_CHANNEL_1, &block_id);
	    assert(ret == 0);

	    ret = mdcom_fifoch_get(MDCOM_CHANNEL_1, block_id, &buffer);
	    assert(ret == 0);

	    syslog(LOG_NOTICE, "MSG: %s", (char *)buffer);

	    syslog(LOG_NOTICE, "Master mdcom_block_free");
	    ret = mdcom_fifoch_free(MDCOM_CHANNEL_1, block_id);
	    assert(ret == 0);


	    /* phase 5 */
	    syslog(LOG_NOTICE, "Master wait for EVEN messages");
	    ret = mdcom_filter_set(MDCOM_CHANNEL_0, MDCOM_FILTER_ID_EVEN);
	    assert(ret == 0);

	    ret = mdcom_fifoch_notify(MDCOM_CHANNEL_0);
	    assert(ret == 0);

	#ifdef	WAITMODE_POLLING
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, 0);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#elif	defined(WAITMODE_TIMEOUT)
		do  {
		    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, TIMEOUT_TIME);
		}
		while( (ret == MDCOM_TIMEOUT) && (app_terminate == false) );
	#else	/* WAITMODE */
	    ret = mdcom_fifoch_wait(MDCOM_CHANNEL_0, -1);
	#endif	/* WAITMODE */
        if( app_terminate == true )  {
            continue;
        }
		if( ret != MDCOM_SUCCESS )
		{
		    syslog(LOG_NOTICE, "ERROR:  mdcom_fifoch_wait() failed.  ret=%d", ret);
			TerminateTask();
		}

        while(app_terminate == false) {
	        ret = mdcom_fifoch_dequeue(MDCOM_CHANNEL_0, &block_id);
	        if (ret == MDCOM_EMPTY) break;
	        assert(ret == 0);

	        ret = mdcom_fifoch_get(MDCOM_CHANNEL_0, block_id, &buffer);
	        assert(ret == 0);

	        syslog(LOG_NOTICE, "number: %u", *((uint32_t *)buffer));

	        ret = mdcom_fifoch_free(MDCOM_CHANNEL_0, block_id);
	        assert(ret == 0);
	    }
        if( app_terminate == true )  {
            continue;
        }

	    syslog(LOG_NOTICE, "Master com done!!");

	    share_mem_test();
    }
}   /* TASK( MainTask ) */


/*
 *  SHMEMテスト
 */
#define SHMEM_SIZE 64

static void
share_mem_test(void)
{
	int ret;
	void *buffer;
	uint32_t size;
	int data;
	int i;

	size = 0;

	syslog(LOG_NOTICE, "Share mem test start!.");

	ret = mdcom_smemch_get(MDCOM_SHMEM_1, &buffer);
	if(ret != 0) {
		syslog(LOG_NOTICE, "Get Buffer Error!! %d",ret);
	}

	syslog(LOG_NOTICE, "buffer 0x%x, Size %d.", buffer, size);

	data = 0;
    while(app_terminate == false) {
		for(i = 0; i < SHMEM_SIZE; i++){
			*((uint32_t *)buffer + i) = data + i;
		}
		syslog(LOG_NOTICE, "Write data %d to %d", data, data + SHMEM_SIZE);
		data++;

		SetRelAlarm(WakeTaskArm, 400 * TICK_FOR_10MS, 0);
		WaitEvent(WakeTaskEvt);
		ClearEvent(WakeTaskEvt);
	}
}


/*
 *  定期実行タスク
 */
TASK(PeriodicTask)
{
	static int i = 0;

	syslog(LOG_EMERG, "task %d", i++);
	TerminateTask();
}


/*
 * MDCOM 終了フックルーチン
 */
void
MdcomTermHook(uint32_t cause)
{
	syslog(LOG_NOTICE, "***** MDCOM terminate (cause %d) *****\n", cause);
    app_terminate = true;
}


/*
 *  エラーフックルーチン
 */
#ifdef CFG_USE_ERRORHOOK
void
ErrorHook(StatusType Error)
{
	/*
	 *  エラー要因ごとのパラメータログ出力
	 */
	switch (OSErrorGetServiceId()) {
	case OSServiceId_ActivateTask:
		syslog(LOG_INFO, "Error:%s=ActivateTask(%d)", atk2_strerror(Error), OSError_ActivateTask_TaskID());
		break;
	case OSServiceId_TerminateTask:
		syslog(LOG_INFO, "Error:%s=TerminateTask()", atk2_strerror(Error));
		break;
	case OSServiceId_ChainTask:
		syslog(LOG_INFO, "Error:%s=ChainTask(%d)", atk2_strerror(Error), OSError_ChainTask_TaskID());
		break;
	case OSServiceId_Schedule:
		syslog(LOG_INFO, "Error:%s=Schedule()", atk2_strerror(Error));
		break;
	case OSServiceId_GetTaskID:
		syslog(LOG_INFO, "Error:%s=GetTaskID(0x%p)", atk2_strerror(Error), OSError_GetTaskID_TaskID());
		break;
	case OSServiceId_GetTaskState:
		syslog(LOG_INFO, "Error:%s=GetTaskState(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetTaskState_TaskID(), OSError_GetTaskState_State());
		break;
	case OSServiceId_EnableAllInterrupts:
		syslog(LOG_INFO, "Error:%s=EnableAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_DisableAllInterrupts:
		syslog(LOG_INFO, "Error:%s=DisableAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_ResumeAllInterrupts:
		syslog(LOG_INFO, "Error:%s=ResumeAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_SuspendAllInterrupts:
		syslog(LOG_INFO, "Error:%s=SuspendAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_ResumeOSInterrupts:
		syslog(LOG_INFO, "Error:%s=ResumeOSInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_SuspendOSInterrupts:
		syslog(LOG_INFO, "Error:%s=SuspendOSInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_GetISRID:
		syslog(LOG_INFO, "Error:%s=GetISRID()", atk2_strerror(Error));
		break;
	case OSServiceId_GetResource:
		syslog(LOG_INFO, "Error:%s=GetResource(%d)", atk2_strerror(Error), OSError_GetResource_ResID());
		break;
	case OSServiceId_ReleaseResource:
		syslog(LOG_INFO, "Error:%s=ReleaseResource(%d)", atk2_strerror(Error), OSError_ReleaseResource_ResID());
		break;
	case OSServiceId_SetEvent:
		syslog(LOG_INFO, "Error:%s=SetEvent(%d, 0x%x)", atk2_strerror(Error),
			   OSError_SetEvent_TaskID(), OSError_SetEvent_Mask());
		break;
	case OSServiceId_ClearEvent:
		syslog(LOG_INFO, "Error:%s=ClearEvent(0x%x)", atk2_strerror(Error), OSError_ClearEvent_Mask());
		break;
	case OSServiceId_GetEvent:
		syslog(LOG_INFO, "Error:%s=GetEvent(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetEvent_TaskID(), OSError_GetEvent_Event());
		break;
	case OSServiceId_WaitEvent:
		syslog(LOG_INFO, "Error:%s=WaitEvent(0x%x)", atk2_strerror(Error), OSError_WaitEvent_Mask());
		break;
	case OSServiceId_GetAlarmBase:
		syslog(LOG_INFO, "Error:%s=GetAlarmBase(0x%p)", atk2_strerror(Error), OSError_GetAlarmBase_AlarmID());
		break;
	case OSServiceId_GetAlarm:
		syslog(LOG_INFO, "Error:%s=GetAlarm(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetAlarm_AlarmID(), OSError_GetAlarm_Tick());
		break;
	case OSServiceId_SetRelAlarm:
		syslog(LOG_INFO, "Error:%s=SetRelAlarm(%d, %d, %d)", atk2_strerror(Error),
			   OSError_SetRelAlarm_AlarmID(), OSError_SetRelAlarm_increment(), OSError_SetRelAlarm_cycle());
		break;
	case OSServiceId_SetAbsAlarm:
		syslog(LOG_INFO, "Error:%s=SetAbsAlarm(%d, %d, %d)", atk2_strerror(Error),
			   OSError_SetAbsAlarm_AlarmID(), OSError_SetAbsAlarm_start(), OSError_SetAbsAlarm_cycle());
		break;
	case OSServiceId_CancelAlarm:
		syslog(LOG_INFO, "Error:%s=CancelAlarm(%d)", atk2_strerror(Error), OSError_CancelAlarm_AlarmID());
		break;
	case OSServiceId_StartScheduleTableRel:
		syslog(LOG_INFO, "Error:%s=StartScheduleTableRel(%d, %d)", atk2_strerror(Error),
			   OSError_StartScheduleTableRel_ScheduleTableID(), OSError_StartScheduleTableRel_Offset());
		break;
	case OSServiceId_StartScheduleTableAbs:
		syslog(LOG_INFO, "Error:%s=StartScheduleTableAbs(%d, %d)", atk2_strerror(Error),
			   OSError_StartScheduleTableAbs_ScheduleTableID(), OSError_StartScheduleTableAbs_Start());
		break;
	case OSServiceId_StopScheduleTable:
		syslog(LOG_INFO, "Error:%s=StopScheduleTable(%d)", atk2_strerror(Error), OSError_StopScheduleTable_ScheduleTableID());
		break;
	case OSServiceId_NextScheduleTable:
		syslog(LOG_INFO, "Error:%s=NextScheduleTable(%d, %d)", atk2_strerror(Error),
			   OSError_NextScheduleTable_ScheduleTableID_From(), OSError_NextScheduleTable_ScheduleTableID_To());
		break;
	case OSServiceId_GetScheduleTableStatus:
		syslog(LOG_INFO, "Error:%s=GetScheduleTableStatus(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetScheduleTableStatus_ScheduleTableID(), OSError_GetScheduleTableStatus_ScheduleStatus());
		break;
	case OSServiceId_GetActiveApplicationMode:
		syslog(LOG_INFO, "Error:%s=GetActiveApplicationMode()", atk2_strerror(Error));
		break;
	case OSServiceId_StartCore:
		syslog(LOG_INFO, "Error:%s=StartCore(%d, %d)", atk2_strerror(Error), OSError_StartCore_CoreID(), OSError_StartCore_Status());
		break;
	case OSServiceId_StartNonAutosarCore:
		syslog(LOG_INFO, "Error:%s=StartNonAutosarCore(%d, %d)", atk2_strerror(Error), OSError_StartNonAutosarCore_CoreID(), OSError_StartNonAutosarCore_Status());
		break;
	case OSServiceId_StartOS:
		syslog(LOG_INFO, "Error:%s=StartOS(%d)", atk2_strerror(Error), OSError_StartOS_Mode());
		break;
	case OSServiceId_ShutdownAllCores:
		syslog(LOG_INFO, "Error:%s=ShutdownAllCores()", atk2_strerror(Error));
		break;
	case OSServiceId_IncrementCounter:
		syslog(LOG_INFO, "Error:%s=IncrementCounter(%d)", atk2_strerror(Error), OSError_IncrementCounter_CounterID());
		break;
	case OSServiceId_GetSpinlock:
		syslog(LOG_INFO, "Error:%s=GetSpinlock(%d)", atk2_strerror(Error), OSError_GetSpinlock_SpinlockId());
		break;
	case OSServiceId_ReleaseSpinlock:
		syslog(LOG_INFO, "Error:%s=ReleaseSpinlock(%d)", atk2_strerror(Error), OSError_ReleaseSpinlock_SpinlockId());
		break;
	case OSServiceId_TryToGetSpinlock:
		syslog(LOG_INFO, "Error:%s=TryToGetSpinlock(%d)", atk2_strerror(Error), OSError_GetSpinlock_SpinlockId());
		break;
	case OSServiceId_TaskMissingEnd:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_strerror(Error));
		break;
	case IOCServiceId_IOC_Send:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Write:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_SendGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_WriteGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Receive:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Read:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_ReceiveGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_ReadGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_EmptyQueue:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_send_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_write_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_receive_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_read_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	default:
		syslog(LOG_INFO, "Error:%s=UnKnownFunc()", atk2_strerror(Error));
		break;
	}

}   /* ErrorHook */
#endif /* CFG_USE_ERRORHOOK */


/*
 *  プレタスクフックルーチン
 *
 *  空ルーチンを呼出す
 */
#ifdef CFG_USE_PRETASKHOOK
void
PreTaskHook(void)
{
}   /* PreTaskHook */
#endif /* CFG_USE_PRETASKHOOK */


/*
 *  ポストタスクフックルーチン
 *
 *  空ルーチンを呼出す
 */
#ifdef CFG_USE_POSTTASKHOOK
void
PostTaskHook(void)
{
}   /* PostTaskHook */
#endif /* CFG_USE_POSTTASKHOOK */


/*
 *  スタートアップフックルーチン
 */
#ifdef CFG_USE_STARTUPHOOK
#ifdef TOPPERS_ENABLE_SYS_TIMER
extern void target_timer_initialize(void);
#endif /* TOPPERS_ENABLE_SYS_TIMER */

void
StartupHook(void)
{
	CoreIdType coreid = GetCoreID();

#ifdef TOPPERS_ENABLE_SYS_TIMER
	target_timer_initialize();
#endif /* TOPPERS_ENABLE_SYS_TIMER */
	if (coreid == OS_CORE_ID_MASTER) {
		syslog_initialize();
		syslog_msk_log(LOG_UPTO(LOG_INFO));
		InitSerial();
		print_banner();
	}
	else {
		InitSerial();
	}
	syslog(LOG_EMERG, "StartupHook @ core%d", coreid);
}   /* StartupHook */
#endif /* CFG_USE_STARTUPHOOK */


/*
 *  シャットダウンフックルーチン
 */
#ifdef CFG_USE_SHUTDOWNHOOK
#ifdef TOPPERS_ENABLE_SYS_TIMER
extern void target_timer_terminate(void);
#endif /* TOPPERS_ENABLE_SYS_TIMER */

void
ShutdownHook(StatusType Error)
{
	/* 終了ログ出力 */
	syslog(LOG_INFO, "");
	syslog(LOG_INFO, "Sample System ShutDown");
	syslog(LOG_INFO, "ShutDownCode:%s", atk2_strerror(Error));
	syslog(LOG_INFO, "");

	if (Error == E_OS_SYS_ASSERT_FATAL) {
		syslog(LOG_INFO, "fatal_file_name:%s", kernel_fatal_file_name);
		syslog(LOG_INFO, "fatal_line_num:%d", kernel_fatal_line_num);
	}

#ifdef TOPPERS_ENABLE_SYS_TIMER
	target_timer_terminate();
#endif /* TOPPERS_ENABLE_SYS_TIMER */
	TermSerial();

}   /* ShutdownHook */
#endif /* CFG_USE_SHUTDOWNHOOK */


/*
 *  プロテクションフックルーチン
 */
#ifdef CFG_USE_PROTECTIONHOOK
ProtectionReturnType
ProtectionHook(StatusType FatalError)
{
	StatusType ercd;

	syslog(LOG_INFO, "");
	syslog(LOG_INFO, "ProtectionHook");

	if (FatalError == E_OS_STACKFAULT) {
		syslog(LOG_INFO, "E_OS_STACKFAULT");
		ercd = PRO_SHUTDOWN;
	}
	else if (FatalError == E_OS_PROTECTION_EXCEPTION) {
		syslog(LOG_INFO, "E_OS_PROTECTION_EXCEPTION");
		ercd = PRO_IGNORE;
	}
	else {
		ercd = PRO_SHUTDOWN;
	}

	return(ercd);
}
#endif /* CFG_USE_PROTECTIONHOOK */


/*
 *  ICI ISRs
 */
ICISR(Core0_ICI_1) {

	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_1 run!");
	syslog(LOG_INFO, "Coer0_ICI_1 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_1 GetApplicationID() = %d", GetApplicationID());
	RaiseInterCoreInterrupt(Core1_ICI_2);
}

ICISR(Core1_ICI_1) {
	syslog(LOG_INFO, "/***********************/\nCoer1_ICI_1 run!");
	syslog(LOG_INFO, "Coer1_ICI_1 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer1_ICI_1 GetApplicationID() = %d", GetApplicationID());
	RaiseInterCoreInterrupt(Core0_ICI_2);
}

ICISR(Core0_ICI_2) {
	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_2 run!");
	syslog(LOG_INFO, "Coer0_ICI_2 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_2 GetApplicationID() = %d", GetApplicationID());
}

ICISR(Core1_ICI_2) {
	syslog(LOG_INFO, "/***********************/\nCoer1_ICI_2 run!");
	syslog(LOG_INFO, "Coer1_ICI_2 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer1_ICI_2 GetApplicationID() = %d", GetApplicationID());
}

ICISR(Core0_ICI_3) {
	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_3 run!");
	syslog(LOG_INFO, "Coer0_ICI_3 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_3 GetApplicationID() = %d", GetApplicationID());
}
