#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "kernel_cfg.h"
#include "sample1.h"
#include "sil.h"
#include "syssvc/logtask.h"
#include <string.h>

#include "mdcom.h"
#include "mdcom_config.h"

#ifdef TOPPERS_SAFEG_SECURE

#include "syscalls_api.h"

void btask(intptr_t exinf)
{
	uint32_t ret;
	uint32_t nt_start_address;

	syslog(LOG_NOTICE, "BTASK for Core%d Starts", exinf);

	while((nt_start_address = target_get_nt_start_address()) == -1);

	syslog(LOG_NOTICE, "%d : SafeG First switch to NT : address 0x%x", exinf, nt_start_address);
	ret = safeg_syscall_setntpc(nt_start_address);

	if (ret != SAFEG_SYSCALL_ERROR__OK) {
		syslog(LOG_NOTICE, "%d : SafeG ERROR: setntpc %u", exinf, ret);
		return;
	}

	while(1) {
		SIL_PRE_LOC;

		SIL_LOC_INT();

		ret = safeg_syscall_switch();
		SIL_UNL_INT();

		if (ret != SAFEG_SYSCALL_ERROR__OK) {
			syslog(LOG_NOTICE, "%d : SafeG ERROR: switch %u", exinf, ret);
			break;
		}
	}
}
#endif /* TOPPERS_SAFEG_SECURE */

void main_task()
{
	int ret;
	mdcom_id_t block_id;
	void *p_block;
	char msg[] = "hello world";

	logtask_flush(0);
	syslog(LOG_NOTICE, "MDCOM fifo test program starts.");

	/* DualOS-COM 初期化 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_init.");
	ret = mdcom_init(0x0, -1);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_init. ret = %d", ret);
	
	/* FIFO ポインタ取得 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_fifoch_alloc.");
	ret = mdcom_fifoch_alloc(MDCOM_FIFOCH_0, &block_id);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_alloc. ret = %d", ret);

	/* FIFO ブロック取得 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_fifoch_get.");
	ret = mdcom_fifoch_get(MDCOM_FIFOCH_0, block_id, &p_block);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_get. ret = %d", ret);

	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] Send messag %s", msg);
	
	/* FIFO ブロックへ書込み */
	memcpy(p_block, (void *)msg, sizeof(msg));

	/* FIFO へ積む */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_fifoch_enqueue.");
	ret = mdcom_fifoch_enqueue(MDCOM_FIFOCH_0, block_id);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_enqueue. ret = %d", ret);

	/* スレーブ側へ通知 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_fifoch_notify.");
	ret = mdcom_fifoch_notify(MDCOM_FIFOCH_0);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_notify. ret = %d", ret);

	/* Ack 待ち */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_fifoch_wait.");
	ret = mdcom_fifoch_wait(MDCOM_FIFOCH_0, -1);
	assert(ret == 0);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_fifoch_wait. ret = %d", ret);

    syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] test completed.");
}
