#include <kernel.h>
#include <t_syslog.h>
#include <t_stdlib.h>
#include "syssvc/serial.h"
#include "syssvc/syslog.h"
#include "syssvc/logtask.h"
#include "kernel_cfg.h"
#include "sample1.h"
#include "sil.h"

#include "mdcom.h"
#include "mdcom_config.h"

#define	SMEMCH_ID1	0

#ifdef TOPPERS_SAFEG_SECURE

#include "syscalls_api.h"

void btask(intptr_t exinf)
{
	uint32_t ret;
	uint32_t nt_start_address;

	syslog(LOG_NOTICE, "BTASK for Core%d Starts", exinf);

	while((nt_start_address = target_get_nt_start_address()) == -1);

	syslog(LOG_NOTICE, "%d : SafeG First switch to NT : address 0x%x", exinf, nt_start_address);
	ret = safeg_syscall_setntpc(nt_start_address);

	if (ret != SAFEG_SYSCALL_ERROR__OK) {
		syslog(LOG_NOTICE, "%d : SafeG ERROR: setntpc %u", exinf, ret);
		return;
	}

	while(1) {
		SIL_PRE_LOC;

		SIL_LOC_INT();

		ret = safeg_syscall_switch();
		SIL_UNL_INT();

		if (ret != SAFEG_SYSCALL_ERROR__OK) {
			syslog(LOG_NOTICE, "%d : SafeG ERROR: switch %u", exinf, ret);
			break;
		}
	}
}
#endif /* TOPPERS_SAFEG_SECURE */

void main_task()
{
	int *p_smemch1, *p_smemch2;
	int ret;
	int i;

	logtask_flush(0);
	syslog(LOG_NOTICE, "MDCOM shmem test program starts.");

	/* DualOS-COM 初期化 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_init.");
	ret = mdcom_init(0x0, -1);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_init. ret = %x", ret);
	
	/* 共有メモリポインタ取得 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start mdcom_smemch_get.");
	ret = mdcom_smemch_get(SMEMCH_ID1, (void **)(&p_smemch1));
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   mdcom_smemch_get. ret = %x", ret);

#if 1
	/* ロック取得 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start trylock.");
	do {
		ret = mdcom_smemch_trylock(SMEMCH_ID1);
	} while (ret == MDCOM_LOCKED);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   trylock.");

	/* 共有メモリへ書込み */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start shared memory write.");
	for (i = 0; i < 16; i++) {
		p_smemch1[i] = i;
	}
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   shared memory write.");

	/* ロック開放 */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start unlock.");
	ret = mdcom_smemch_unlock(SMEMCH_ID1);
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   unlock. ret = %d", ret);
#endif

	/* 共有メモリ読込み */
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] start shared memory read.");
	for (i = 0; i < 16; i++) {
		syslog(LOG_NOTICE, "p_smemch1[%2d] = %8x", i, p_smemch1[i]);
	}
	syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] end   shared memory read.");

    syslog(LOG_NOTICE, "["MDCOM_DOMAIN_NAME"] test completed.");
}
