/*
 *  TOPPERS ATK2
 *      Toyohashi Open Platform for Embedded Real-Time Systems
 *      Automotive Kernel Version 2
 *
 *  Copyright (C) 2000-2003 by Embedded and Real-Time Systems Laboratory
 *                              Toyohashi Univ. of Technology, JAPAN
 *  Copyright (C) 2004-2019 by Center for Embedded Computing Systems
 *              Graduate School of Information Science, Nagoya Univ., JAPAN
 *  Copyright (C) 2011-2017 by FUJI SOFT INCORPORATED, JAPAN
 *  Copyright (C) 2011-2013 by Spansion LLC, USA
 *  Copyright (C) 2011-2017 by NEC Communication Systems, Ltd., JAPAN
 *  Copyright (C) 2011-2016 by Panasonic Advanced Technology Development Co., Ltd., JAPAN
 *  Copyright (C) 2011-2014 by Renesas Electronics Corporation, JAPAN
 *  Copyright (C) 2011-2016 by Sunny Giken Inc., JAPAN
 *  Copyright (C) 2011-2017 by TOSHIBA CORPORATION, JAPAN
 *  Copyright (C) 2004-2017 by Witz Corporation
 *  Copyright (C) 2014-2016 by AISIN COMCRUISE Co., Ltd., JAPAN
 *  Copyright (C) 2014-2016 by eSOL Co.,Ltd., JAPAN
 *  Copyright (C) 2014-2017 by SCSK Corporation, JAPAN
 *  Copyright (C) 2015-2017 by SUZUKI MOTOR CORPORATION
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，AUTOSAR（AUTomotive Open System ARchitecture）仕
 *  様に基づいている．上記の許諾は，AUTOSARの知的財産権を許諾するもので
 *  はない．AUTOSARは，AUTOSAR仕様に基づいたソフトウェアを商用目的で利
 *  用する者に対して，AUTOSARパートナーになることを求めている．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  $Id$
 */

#include "Os.h"
#include "t_syslog.h"
#include "t_stdlib.h"
#include "sysmod/serial.h"
#include "sysmod/syslog.h"
#include "sample1_m_atk2.h"
#include "tmout_test.h"

#include "sysmod/banner.h"
#include "target_sysmod.h"
#include "target_serial.h"
#include "sysmod/serial.h"
#include "target_hw_counter.h"
#include "mdcom.h"
#include "mdcom_config.h"
#include "arm.h"


/*
 *	コンフィグレーション
 */
/* タイムアウト時間 [ms] */
	/* mdcom_fifoch_wait() - タイムアウト時間満了テスト */
#define	TMO_INIT_EXPIRED		100
	/* mdcom_fifoch_wait() - タイムアウトモードテスト */
#define	TMO_INIT				3000
	/* mdcom_fifoch_wait() - 複数チャネル同時動作 タイムアウトモードでのイベント受信テスト */
#define	TMO_WAIT_FIFOCH_1		200
#define	TMO_WAIT_FIFOCH_2		500
#define	TMO_WAIT_FIFOCH_3		400
#define	TMO_WAIT_FIFOCH_4		600
#define	TMO_WAIT_FIFOCH_5		300
	/* mdcom_fifoch_wait() - 複数チャネル同時動作 タイムアウト時間満了テスト */
#define	TMO_EXPIRED_FIFOCH_1	400
#define	TMO_EXPIRED_FIFOCH_2	100
#define	TMO_EXPIRED_FIFOCH_3	500
#define	TMO_EXPIRED_FIFOCH_4	300
#define	TMO_EXPIRED_FIFOCH_5	200

/* mdcom_fifoch_wait() - 複数チャネル同時動作 繰り返しテスト回数 */
#define TESTCOUNT_FIFOCH		10

/* デバッグ用ログ出力 */
//#define	MDCOM_DEBUG


/*
 *  定数・型定義
 */
#define	TICK_FOR_1MS		(TICK_FOR_10MS / 10)

/* FIFOCHテストタスク 管理データ */
typedef struct
{
	mdcom_id_t		fifoch_id;
	mdcom_time_t	timeout;
	mdcom_time_t	elapse_time;
	int				ercd;
	EventMaskType	event;
} REQ_FIFOCH;

/* テストエラーコード */
#define	TEST_RETCD_SUCCESS			0
#define	TEST_RETCD_FAILED_ITEM1		-1
#define	TEST_RETCD_FAILED_ITEM2		-2
#define	TEST_RETCD_FAILED_ITEM3		-3
#define	TEST_RETCD_FAILED_ITEM4		-4
#define	TEST_RETCD_FAILED_ITEM5		-5
#define	TEST_RETCD_FAILED_ITEM6		-6
#define	TEST_RETCD_FAILED_ITEM7		-7
#define	TEST_RETCD_FAILED_ITEM8		-8
#define	TEST_RETCD_FAILED_ITEM9		-9
#define	TEST_RETCD_FAILED_ITEM10	-10
#define	TEST_RETCD_FAILED_ITEM11	-11
#define	TEST_RETCD_FAILED_ITEM12	-12
#define	TEST_RETCD_FAILED_ITEM13	-13
#define	TEST_RETCD_FAILED_ITEM14	-14
#define	TEST_RETCD_FAILED_ITEM15	-15
#define	TEST_RETCD_FAILED_ITEM16	-16
#define	TEST_RETCD_FAILED_ITEM17	-17
#define	TEST_RETCD_FAILED_ITEM18	-18
#define	TEST_RETCD_FAILED_ITEM19	-19
#define	TEST_RETCD_FAILED_ITEM20	-20
#define	TEST_RETCD_ERR_OTHER		-9999


/*
 *  グローバル変数
 */
/* MDCOMアプリ停止要求 */
bool_t      app_terminate;
/* テスト実施タスク タスクID */
TaskType	test_task_id;

/* FIFOCHタスク 動作指定 */
REQ_FIFOCH	req_fifoch[MDCOM_NUM_FIFOCHS];

/* 文字列 */
const char	str_passed[] = "  ----- Pass -----\n";
const char	str_failed[] = "  ----- Failed -----\n";


/*
 *  関数プロトタイプ宣言
 */
/* Task */
TASK(MainTask);
TASK(FifochTask0);
TASK(FifochTask1);
TASK(FifochTask2);
TASK(FifochTask3);
TASK(FifochTask4);

/* External */
sint32 main(void);
extern void		*memcpy(void *dest, const void *src, uint32_t len);

/* Internal */
static int		TmoutTest_MdcomInit(void);
static int		TmoutTest_MdcomFicochWait(void);
static int		MultChannelsEventWaitTest(void);
static int		MultChannelsTimeoutExpiredTest(void);
static void		fifoch_test(REQ_FIFOCH *req);
static int		sync_remote_app(void);
static int		restart_remote_app(void);
static TESTCMD	*send_testcmd(uint32_t cmd, uint32_t param1, uint32_t param2, uint32_t param3, uint32_t param4, uint32_t param5);
static void		lock_testcmd(void);
static void		unlock_testcmd(void);
static void		wait_to_push_ant_key(void);
#ifdef MDCOM_DEBUG
static int  	LOG_mdcom_init(mdcom_mask_t fifoch_mask, mdcom_time_t timeout);
static int  	LOG_mdcom_sense_process(uint32_t *p_num_process);
static int  	LOG_mdcom_smemch_get(const mdcom_id_t smemch_id, void **buffer);
static int  	LOG_mdcom_smemch_trylock(const mdcom_id_t smemch_id);
static int  	LOG_mdcom_smemch_unlock(const mdcom_id_t smemch_id);
static int  	LOG_mdcom_sense_fifoch(mdcom_mask_t *p_fifoch_mask);
static int  	LOG_mdcom_fifoch_alloc(const mdcom_id_t fifoch_id, mdcom_id_t *block_id);
static int  	LOG_mdcom_fifoch_free(const mdcom_id_t fifoch_id, const mdcom_id_t block_id);
static int  	LOG_mdcom_fifoch_get(const mdcom_id_t fifoch_id, const mdcom_id_t block_id, void **buffer);
static int  	LOG_mdcom_fifoch_enqueue(const mdcom_id_t fifoch_id, const mdcom_id_t block_id);
static int  	LOG_mdcom_fifoch_dequeue(const mdcom_id_t fifoch_id, mdcom_id_t *block_id);
static int  	LOG_mdcom_fifoch_notify(const mdcom_id_t fifoch_id);
static int  	LOG_mdcom_fifoch_wait(const mdcom_id_t fifoch_id, const mdcom_time_t timeout);
static int  	LOG_mdcom_filter_set(const mdcom_id_t fifoch_id, const mdcom_id_t  filter_id);
static int  	LOG_mdcom_fifoch_init(const mdcom_id_t fifoch_id);
#endif	/* MDCOM_DEBUG */


/*
 *  マクロ
 */
#define GetAppModeInfo()	(0)

/* ログ出力用APIラッパ */
#ifdef MDCOM_DEBUG
#define	Log_mdcom_init				LOG_mdcom_init
#define Log_mdcom_smemch_get		LOG_mdcom_smemch_get
#define Log_mdcom_smemch_trylock	LOG_mdcom_smemch_trylock
#define Log_mdcom_smemch_unlock		LOG_mdcom_smemch_unlock
#define Log_mdcom_sense_fifoch		LOG_mdcom_sense_fifoch
#define Log_mdcom_fifoch_alloc		LOG_mdcom_fifoch_alloc
#define Log_mdcom_fifoch_free		LOG_mdcom_fifoch_free
#define Log_mdcom_fifoch_get		LOG_mdcom_fifoch_get
#define Log_mdcom_fifoch_enqueue	LOG_mdcom_fifoch_enqueue
#define Log_mdcom_fifoch_dequeue	LOG_mdcom_fifoch_dequeue
#define Log_mdcom_fifoch_notify		LOG_mdcom_fifoch_notify
#define Log_mdcom_fifoch_wait		LOG_mdcom_fifoch_wait
#define Log_mdcom_filter_set		LOG_mdcom_filter_set
#define Log_mdcom_fifoch_init		LOG_mdcom_fifoch_init
#else	/* MDCOM_DEBUG */
#define	Log_mdcom_init				mdcom_init
#define Log_mdcom_smemch_get		mdcom_smemch_get
#define Log_mdcom_smemch_trylock	mdcom_smemch_trylock
#define Log_mdcom_smemch_unlock		mdcom_smemch_unlock
#define Log_mdcom_sense_fifoch		mdcom_sense_fifoch
#define Log_mdcom_fifoch_alloc		mdcom_fifoch_alloc
#define Log_mdcom_fifoch_free		mdcom_fifoch_free
#define Log_mdcom_fifoch_get		mdcom_fifoch_get
#define Log_mdcom_fifoch_enqueue	mdcom_fifoch_enqueue
#define Log_mdcom_fifoch_dequeue	mdcom_fifoch_dequeue
#define Log_mdcom_fifoch_notify		mdcom_fifoch_notify
#define Log_mdcom_fifoch_wait		mdcom_fifoch_wait
#define Log_mdcom_filter_set		mdcom_filter_set
#define Log_mdcom_fifoch_init		mdcom_fifoch_init
#endif	/* MDCOM_DEBUG */


/*
 *  ユーザメイン関数
 *
 *  アプリケーションモードの判断と，カーネル起動
 */
sint32
main(void)
{
	AppModeType	crt_app_mode;
	StatusType	ercd;
	CoreIdType	i;

	/*
	 *  アプリケーションモードの判断
	 */
	switch (GetAppModeInfo()) {
	case 0:
		crt_app_mode = AppMode1;
		break;
	case 1:
		crt_app_mode = AppMode2;
		break;
	default:
		crt_app_mode = AppMode3;
		break;
	}

	if (GetCoreID() == OS_CORE_ID_MASTER) {
		for (i = 0; i < TNUM_HWCORE; i++) {
			if (i != OS_CORE_ID_MASTER) {
				StartCore(i, &ercd);
			}
		}
		/*
		 *  カーネル起動
		 */
		StartOS(crt_app_mode);
	}
	else {
		/*
		 *  カーネル起動
		 */
		StartOS(DONOTCARE);
	}


	while (1) {
	}
}   /* main */


/*
 *  メインタスク
 */
TASK(MainTask)
{
	TESTCMD		*testcmd;
    int		ret;

    syslog_msk_log(LOG_UPTO(LOG_DEBUG));

    do
    {
        app_terminate = false;

	    syslog(LOG_NOTICE, "MDCOM timeout test program starts.");

		/*
		 *  mdcom_init() test
		 */
		ret = TmoutTest_MdcomInit();
		if( ret < 0 )  {
			goto	test_failed;
		}
		if( app_terminate == true )  {
			continue;
		}

		/*
		 *  mdcom_fifoch_wait() test
		 */
		ret = TmoutTest_MdcomFicochWait();
		if( ret < 0 )  {
			goto	test_failed;
		}
		if( app_terminate == true )  {
			continue;
		}

	    syslog(LOG_NOTICE, "****** Test passed ******");
		break;

	test_failed:
	    syslog(LOG_NOTICE, "****** Test failed ******");
		break;
    }
    while( app_terminate == true );

	/* リモートアプリ終了 */
	testcmd = send_testcmd(TESTCMD_REQ_APP_EXIT, 0, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	TerminateTask();
}   /* TASK( MainTask ) */


/*
 *  mdcom_init() test
 */
static int
TmoutTest_MdcomInit(void)
{
	TickType	crnt_tick, elapse_tick;
	uint32_t	elapse_time;
    int			ret, mdcom_retcd;

	syslog(LOG_NOTICE, "***** mdcom_init() timeout test *****\n");

	/*
     *  タイムアウト時間満了 テスト
     */
	syslog(LOG_NOTICE, "[4] Timeout expired test\n");

	/* mdcom_init()実行, 実行時間計測 */
	GetCounterValue(MAIN_HW_COUNTER_CORE0, &crnt_tick);
    mdcom_retcd = Log_mdcom_init(0x0000001F, TMO_INIT_EXPIRED);
	GetElapsedValue(MAIN_HW_COUNTER_CORE0, &crnt_tick, &elapse_tick);
	if( mdcom_retcd != MDCOM_TIMEOUT )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM4;
	}

	/* 1[ms]単位に変換 (小数点以下繰り上げ) */
	elapse_time = (elapse_tick + (TICK_FOR_1MS - 1)) / TICK_FOR_1MS;

	syslog(LOG_NOTICE, "  Timeout time :         %d", TMO_INIT_EXPIRED);
	syslog(LOG_NOTICE, "  Timeout expired time : %d", elapse_time);

	/* タイムアウト満了時間確認 */
	if( (elapse_time >= TMO_INIT_EXPIRED) &&
        (elapse_time < (TMO_INIT_EXPIRED * 1.2 + 10)) )  {
		syslog(LOG_NOTICE, str_passed);
	}
	else
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM4;
	}

	/*
     *  リモート側 アプリ実行要求メッセージ表示
     */
	syslog(LOG_NOTICE, "Execute MDCOM remote application.");
	wait_to_push_ant_key();

	/*
     *  ブロッキングモード テスト
     */
	syslog(LOG_NOTICE, "[1] MDCOM initialize with blocking mode test\n");

    mdcom_retcd = Log_mdcom_init(0x0000001F, -1);
	if( mdcom_retcd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM1;
	}

	ret = sync_remote_app();
	assert( ret >= 0 );

	ret = restart_remote_app();
	assert( ret >= 0 );

	syslog(LOG_NOTICE, str_passed);

	/*
     *  ポーリングモード テスト
     */
	syslog(LOG_NOTICE, "[2][3] MDCOM initialize with polling mode test\n");

	do  {
	    mdcom_retcd = Log_mdcom_init(0x0000001F, 0);
	}
	while( mdcom_retcd == MDCOM_TIMEOUT );
	if( mdcom_retcd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM2;
	}

	ret = sync_remote_app();
	assert( ret >= 0 );

	ret = restart_remote_app();
	assert( ret >= 0 );

	syslog(LOG_NOTICE, str_passed);

	/*
     *  タイムアウトモード テスト
     */
	syslog(LOG_NOTICE, "[5] MDCOM initialize with timeout mode test\n");

    mdcom_retcd = Log_mdcom_init(0x0000001F, TMO_INIT);
	if( mdcom_retcd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM5;
	}

	ret = sync_remote_app();
	assert( ret >= 0 );

	ret = restart_remote_app();
	assert( ret >= 0 );

	syslog(LOG_NOTICE, str_passed);

	return TEST_RETCD_SUCCESS;
}


/*
 *  mdcom_fifoch_wait() test
 */
static int
TmoutTest_MdcomFicochWait(void)
{
	TESTCMD	*testcmd;
	StatusType	ercd;
	bool_t	test_success;
	int		i, mdcom_retcd, ret;

	GetTaskID(&test_task_id);

	/* MDCOM初期化 */
    mdcom_retcd = Log_mdcom_init(0x3, -1);
	assert(mdcom_retcd == MDCOM_SUCCESS);

	/* リモートアプリとの同期 */
	ret = sync_remote_app();
	assert( ret >= 0 );

	syslog(LOG_NOTICE, "***** mdcom_fifoch_wait() timeout test *****\n");

	/*
     *  1チャネル動作 ブロッキングモード テスト
     */
	syslog(LOG_NOTICE, "[1] 1 channel blocking mode test\n");

	/* テスト実行 */
	req_fifoch[0].timeout = -1;
	ercd = ActivateTask(FifochTask1);
	assert(ercd == E_OK);

	/* リモートアプリへFIFOCHへのNotifyを要求 */
	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_1, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	/* テスト完了待ち */
	ercd = WaitEvent(FifochTask1Evt);
	assert(ercd == E_OK);

	/* 結果確認 */
	if( req_fifoch[0].ercd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM1;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  1チャネル動作 ポーリングモード テスト
     */
	syslog(LOG_NOTICE, "[2][3] 1 channel polling mode test\n");

	/* イベント未受信時の動作確認 */
	mdcom_retcd = Log_mdcom_fifoch_wait(MDCOM_FIFOCH_1, 0);
	if( mdcom_retcd != MDCOM_TIMEOUT )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM2;
	}

	/* リモートアプリへFIFOCHへのNotifyを要求 */
	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_1, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	/* イベント受信時の動作確認 */
	do  {
		mdcom_retcd = Log_mdcom_fifoch_wait(MDCOM_FIFOCH_1, 0);
	}
	while( mdcom_retcd == MDCOM_TIMEOUT );
	if( mdcom_retcd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM3;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  1チャネル動作 タイムアウトモードでのイベント受信テスト
     */
	syslog(LOG_NOTICE, "[4] 1 channel timeout mode test\n");

	/* テスト実行 */
	req_fifoch[0].timeout = 200;
	ercd = ActivateTask(FifochTask1);
	assert(ercd == E_OK);

	/* リモートアプリへFIFOCHへのNotifyを要求 */
	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_1, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	/* テスト完了待ち */
	ercd = WaitEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask1Evt);
	assert(ercd == E_OK);

	/* 結果確認 */
	if( req_fifoch[0].ercd != MDCOM_SUCCESS )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM4;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  1チャネル動作 タイムアウト時間満了テスト
     */
	syslog(LOG_NOTICE, "[5] 1 channel timeout time expired mode test\n");

	/* テスト実行 */
	req_fifoch[0].timeout = 200;
	ercd = ActivateTask(FifochTask1);
	assert(ercd == E_OK);

	/* テスト完了待ち */
	ercd = WaitEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask1Evt);
	assert(ercd == E_OK);

	/* 結果確認 */
	if( (req_fifoch[0].elapse_time >= req_fifoch[0].timeout) &&
        (req_fifoch[0].elapse_time < (req_fifoch[0].timeout * 1.2 + 10)) )  {
		test_success = true;
	}
	else  {
		test_success = false;
	}
	syslog(LOG_NOTICE, "                    Timeout time = %d, Elapse time = %d",
		req_fifoch[0].timeout, req_fifoch[0].elapse_time);

	if( test_success == false )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM5;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  複数チャネル同時動作 タイムアウトモードでのイベント受信テスト
     */
	syslog(LOG_NOTICE, "[6] 5 channels timeout mode test\n");

	ret = MultChannelsEventWaitTest();
	if( ret < 0 )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM6;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  複数チャネル同時動作 タイムアウトモードでのタイムアウト時間満了テスト
     */
	syslog(LOG_NOTICE, "[7] 5 channels timeout time expired test\n");

	ret = MultChannelsTimeoutExpiredTest();
	if( ret < 0 )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM7;
	}

	syslog(LOG_NOTICE, str_passed);

	/*
     *  複数チャネル同時動作 タイムアウトモードでのイベント受信テスト
     *  複数チャネル同時動作 タイムアウトモードでのタイムアウト時間満了テスト
     *  の繰り返し実行テスト
     */
	syslog(LOG_NOTICE, "[8] 5 channels wait event and timeout time expired repeat test\n");

	for( i = 0 ; i < TESTCOUNT_FIFOCH ; i++ )
	{
		syslog(LOG_NOTICE, "  - Wait event test");
		ret = MultChannelsEventWaitTest();
		if( ret < 0 )  {
			break;
		}

		syslog(LOG_NOTICE, "  - Timeout time expired test");
		ret = MultChannelsTimeoutExpiredTest();
		if( ret < 0 )  {
			break;
		}
	}

	if( ret < 0 )
	{
		syslog(LOG_NOTICE, str_failed);
		return TEST_RETCD_FAILED_ITEM8;
	}

	syslog(LOG_NOTICE, str_passed);

	return TEST_RETCD_SUCCESS;
}


/*
 *  複数チャネル同時動作 タイムアウトモードでのイベント受信テスト
 */
static int
MultChannelsEventWaitTest(void)
{
	TESTCMD	*testcmd;
	StatusType	ercd;
	int		i, ret;

	/* テスト実行 */
	req_fifoch[0].timeout = TMO_WAIT_FIFOCH_1;
	ercd = ActivateTask(FifochTask1);
	assert(ercd == E_OK);
	req_fifoch[1].timeout = TMO_WAIT_FIFOCH_2;
	ercd = ActivateTask(FifochTask2);
	assert(ercd == E_OK);
	req_fifoch[2].timeout = TMO_WAIT_FIFOCH_3;
	ercd = ActivateTask(FifochTask3);
	assert(ercd == E_OK);
	req_fifoch[3].timeout = TMO_WAIT_FIFOCH_4;
	ercd = ActivateTask(FifochTask4);
	assert(ercd == E_OK);
	req_fifoch[4].timeout = TMO_WAIT_FIFOCH_5;
	ercd = ActivateTask(FifochTask5);
	assert(ercd == E_OK);

	/* リモートアプリへFIFOCHへのNotifyを要求 */
	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_1, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_2, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_3, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_4, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	testcmd = send_testcmd(TESTCMD_REQ_NOTIFY, MDCOM_FIFOCH_5, 0, 0, 0, 0);
	assert( (testcmd != NULL) && (testcmd->resp == TESTRESP_SUCCESS) );

	/* テスト完了待ち */
	ercd = WaitEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask2Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask2Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask3Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask3Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask4Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask4Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask5Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask5Evt);
	assert(ercd == E_OK);

	/* 結果確認 */
	ret = 0;
	for( i = 0 ; i < MDCOM_NUM_FIFOCHS ; i++ )
	{
		if( req_fifoch[i].ercd == MDCOM_SUCCESS )  {
			syslog(LOG_NOTICE, "  MDCOM_FIFOCH_%d : Pass", i + 1);
		}
		else
		{
			syslog(LOG_NOTICE, "  MDCOM_FIFOCH_%d : Failed", i + 1);
			ret = -1;
		}
	}

	return ret;
}


/*
 *  複数チャネル同時動作 タイムアウトモードでのタイムアウト時間満了テスト
 */
static int
MultChannelsTimeoutExpiredTest(void)
{
	StatusType	ercd;
	int		i, ret;

	/* テスト実行 */
	req_fifoch[0].timeout = TMO_EXPIRED_FIFOCH_1;
	ercd = ActivateTask(FifochTask1);
	assert(ercd == E_OK);
	req_fifoch[1].timeout = TMO_EXPIRED_FIFOCH_2;
	ercd = ActivateTask(FifochTask2);
	assert(ercd == E_OK);
	req_fifoch[2].timeout = TMO_EXPIRED_FIFOCH_3;
	ercd = ActivateTask(FifochTask3);
	assert(ercd == E_OK);
	req_fifoch[3].timeout = TMO_EXPIRED_FIFOCH_4;
	ercd = ActivateTask(FifochTask4);
	assert(ercd == E_OK);
	req_fifoch[4].timeout = TMO_EXPIRED_FIFOCH_5;
	ercd = ActivateTask(FifochTask5);
	assert(ercd == E_OK);

	/* テスト完了待ち */
	ercd = WaitEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask1Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask2Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask2Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask3Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask3Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask4Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask4Evt);
	assert(ercd == E_OK);
	ercd = WaitEvent(FifochTask5Evt);
	assert(ercd == E_OK);
	ercd = ClearEvent(FifochTask5Evt);
	assert(ercd == E_OK);

	/* 結果確認 */
	ret = 0;
	for( i = 0 ; i < MDCOM_NUM_FIFOCHS ; i++ )
	{
		if( (req_fifoch[i].elapse_time >= req_fifoch[i].timeout) &&
            (req_fifoch[i].elapse_time < (req_fifoch[i].timeout * 1.2 + 10)) )  {
			syslog(LOG_NOTICE, "  MDCOM_FIFOCH_%d : Pass", i + 1);
		}
		else
		{
			syslog(LOG_NOTICE, "  MDCOM_FIFOCH_%d : Failed", i + 1);
			ret = -1;
		}
		syslog(LOG_NOTICE, "                    Timeout time = %d, Elapse time = %d",
			req_fifoch[i].timeout, req_fifoch[i].elapse_time);
	}

	return ret;
}


/*
 *  fifoch テストタスク
 */
TASK(FifochTask1)
{
	req_fifoch[0].fifoch_id =   MDCOM_FIFOCH_1;
	req_fifoch[0].event =       FifochTask1Evt;
	fifoch_test(&req_fifoch[0]);

	TerminateTask();
}	/* TASK(FifochTask1) */

TASK(FifochTask2)
{
	req_fifoch[1].fifoch_id =   MDCOM_FIFOCH_2;
	req_fifoch[1].event =       FifochTask2Evt;
	fifoch_test(&req_fifoch[1]);

	TerminateTask();
}	/* TASK(FifochTask2) */

TASK(FifochTask3)
{
	req_fifoch[2].fifoch_id =   MDCOM_FIFOCH_3;
	req_fifoch[2].event =       FifochTask3Evt;
	fifoch_test(&req_fifoch[2]);

	TerminateTask();
}	/* TASK(FifochTask3) */

TASK(FifochTask4)
{
	req_fifoch[3].fifoch_id =   MDCOM_FIFOCH_4;
	req_fifoch[3].event =       FifochTask4Evt;
	fifoch_test(&req_fifoch[3]);

	TerminateTask();
}	/* TASK(FifochTask4) */

TASK(FifochTask5)
{
	req_fifoch[4].fifoch_id =   MDCOM_FIFOCH_5;
	req_fifoch[4].event =       FifochTask5Evt;
	fifoch_test(&req_fifoch[4]);

	TerminateTask();
}	/* TASK(FifochTask5) */


static void		fifoch_test(REQ_FIFOCH *req)
{
	TickType	crnt_tick, elapse_tick;

	req->elapse_time = 0;

	/* mdcom_fifoch_wait()実行, 実行時間計測 */
	GetCounterValue(MAIN_HW_COUNTER_CORE0, &crnt_tick);
	req->ercd = Log_mdcom_fifoch_wait(req->fifoch_id, req->timeout);
	GetElapsedValue(MAIN_HW_COUNTER_CORE0, &crnt_tick, &elapse_tick);

	/* 1[ms]単位に変換 (小数点以下繰り上げ) */
	req->elapse_time = (elapse_tick + (TICK_FOR_1MS - 1)) / TICK_FOR_1MS;

	/* 終了をメインタスクへ通知 */
	SetEvent(test_task_id, req->event);
}


#pragma GCC push_options
#pragma GCC optimize("O0")

/*
 *  リモートアプリとの同期処理
 */
static int
sync_remote_app( void )
{
	TESTCMD	*testcmd;
	volatile uint32_t	resp;
	int		mdcom_retcd;

	mdcom_retcd = Log_mdcom_smemch_get(MDCOM_SHMEMCH_TESTCMD, (void **)&testcmd);
	assert(mdcom_retcd == MDCOM_SUCCESS);

	/* リモートアプリとの同期 */
	lock_testcmd();
	testcmd->resp = TESTRESP_NONE;
	testcmd->cmd =  TESTCMD_READY;
	unlock_testcmd();

	do  {
		resp = testcmd->resp;
	}
	while( resp != TESTRESP_READY );

	return 0;
}


/*
 *  リモートアプリ再起動
 */
static int
restart_remote_app( void )
{
	TESTCMD	*testcmd;
	int		mdcom_retcd;

	mdcom_retcd = Log_mdcom_smemch_get(MDCOM_SHMEMCH_TESTCMD, (void **)&testcmd);
	assert(mdcom_retcd == MDCOM_SUCCESS);

	/* リモートアプリに再起動を要求 */
	lock_testcmd();
	testcmd->cmd =  TESTCMD_REQ_APP_RESTART;
	unlock_testcmd();

	while( app_terminate == false );
	app_terminate = false;

	return 0;
}


/*
 *  テストコマンド実行
 */
static TESTCMD *
send_testcmd(uint32_t cmd, uint32_t param1, uint32_t param2, uint32_t param3, uint32_t param4, uint32_t param5)
{
	TESTCMD *shmemch;
	volatile uint32_t	resp;
	int		mdcom_retcd;

	/* テストコマンド用SHMEMCH ポインタを取得 */
	mdcom_retcd = Log_mdcom_smemch_get(MDCOM_SHMEMCH_TESTCMD, (void **)&shmemch);
	if( mdcom_retcd != MDCOM_SUCCESS )  {
		return NULL;
	}

	/* テストコマンド実行 */
	lock_testcmd();
	shmemch->resp =    TESTRESP_NONE;
	shmemch->param[0] = param1;
	shmemch->param[1] = param2;
	shmemch->param[2] = param3;
	shmemch->param[3] = param4;
	shmemch->param[4] = param5;
	shmemch->cmd =     cmd;
	unlock_testcmd();

	/* 実行完了待ち */
	do  {
		resp = shmemch->resp;
	}
	while( resp == TESTRESP_NONE );

	return shmemch;
}


/*
 *  テスト制御用 shmemch ロック
 */
static void
lock_testcmd(void)
{
	int		mdcom_retcd;

	do  {
		mdcom_retcd = mdcom_smemch_trylock(MDCOM_SHMEMCH_TESTCMD);
	}
	while( mdcom_retcd == MDCOM_LOCKED );
	assert(mdcom_retcd == MDCOM_SUCCESS);
}


/*
 *  テスト制御用 shmemch アンロック
 */
static void
unlock_testcmd(void)
{
	int		mdcom_retcd;

	mdcom_retcd = mdcom_smemch_unlock(MDCOM_SHMEMCH_TESTCMD);
	assert(mdcom_retcd == MDCOM_SUCCESS);
}

#pragma GCC pop_options


/*
 *  キー入力待ち
 */
static void
wait_to_push_ant_key( void )
{
	uint8_t		key;

	syslog(LOG_NOTICE, "***** Push any key *****\n");
	do {
		RecvPolSerialChar(&key);
	}
	while( key == '\0' );
}


/*
 * MDCOM 終了フックルーチン
 */
void
MdcomTermHook(uint32_t cause)
{
	syslog(LOG_NOTICE, "***** MDCOM terminate (cause %d) *****\n", cause);
    app_terminate = true;
}


/*
 *  エラーフックルーチン
 */
#ifdef CFG_USE_ERRORHOOK
void
ErrorHook(StatusType Error)
{
	/*
	 *  エラー要因ごとのパラメータログ出力
	 */
	switch (OSErrorGetServiceId()) {
	case OSServiceId_ActivateTask:
		syslog(LOG_INFO, "Error:%s=ActivateTask(%d)", atk2_strerror(Error), OSError_ActivateTask_TaskID());
		break;
	case OSServiceId_TerminateTask:
		syslog(LOG_INFO, "Error:%s=TerminateTask()", atk2_strerror(Error));
		break;
	case OSServiceId_ChainTask:
		syslog(LOG_INFO, "Error:%s=ChainTask(%d)", atk2_strerror(Error), OSError_ChainTask_TaskID());
		break;
	case OSServiceId_Schedule:
		syslog(LOG_INFO, "Error:%s=Schedule()", atk2_strerror(Error));
		break;
	case OSServiceId_GetTaskID:
		syslog(LOG_INFO, "Error:%s=GetTaskID(0x%p)", atk2_strerror(Error), OSError_GetTaskID_TaskID());
		break;
	case OSServiceId_GetTaskState:
		syslog(LOG_INFO, "Error:%s=GetTaskState(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetTaskState_TaskID(), OSError_GetTaskState_State());
		break;
	case OSServiceId_EnableAllInterrupts:
		syslog(LOG_INFO, "Error:%s=EnableAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_DisableAllInterrupts:
		syslog(LOG_INFO, "Error:%s=DisableAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_ResumeAllInterrupts:
		syslog(LOG_INFO, "Error:%s=ResumeAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_SuspendAllInterrupts:
		syslog(LOG_INFO, "Error:%s=SuspendAllInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_ResumeOSInterrupts:
		syslog(LOG_INFO, "Error:%s=ResumeOSInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_SuspendOSInterrupts:
		syslog(LOG_INFO, "Error:%s=SuspendOSInterrupts()", atk2_strerror(Error));
		break;
	case OSServiceId_GetISRID:
		syslog(LOG_INFO, "Error:%s=GetISRID()", atk2_strerror(Error));
		break;
	case OSServiceId_GetResource:
		syslog(LOG_INFO, "Error:%s=GetResource(%d)", atk2_strerror(Error), OSError_GetResource_ResID());
		break;
	case OSServiceId_ReleaseResource:
		syslog(LOG_INFO, "Error:%s=ReleaseResource(%d)", atk2_strerror(Error), OSError_ReleaseResource_ResID());
		break;
	case OSServiceId_SetEvent:
		syslog(LOG_INFO, "Error:%s=SetEvent(%d, 0x%x)", atk2_strerror(Error),
			   OSError_SetEvent_TaskID(), OSError_SetEvent_Mask());
		break;
	case OSServiceId_ClearEvent:
		syslog(LOG_INFO, "Error:%s=ClearEvent(0x%x)", atk2_strerror(Error), OSError_ClearEvent_Mask());
		break;
	case OSServiceId_GetEvent:
		syslog(LOG_INFO, "Error:%s=GetEvent(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetEvent_TaskID(), OSError_GetEvent_Event());
		break;
	case OSServiceId_WaitEvent:
		syslog(LOG_INFO, "Error:%s=WaitEvent(0x%x)", atk2_strerror(Error), OSError_WaitEvent_Mask());
		break;
	case OSServiceId_GetAlarmBase:
		syslog(LOG_INFO, "Error:%s=GetAlarmBase(0x%p)", atk2_strerror(Error), OSError_GetAlarmBase_AlarmID());
		break;
	case OSServiceId_GetAlarm:
		syslog(LOG_INFO, "Error:%s=GetAlarm(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetAlarm_AlarmID(), OSError_GetAlarm_Tick());
		break;
	case OSServiceId_SetRelAlarm:
		syslog(LOG_INFO, "Error:%s=SetRelAlarm(%d, %d, %d)", atk2_strerror(Error),
			   OSError_SetRelAlarm_AlarmID(), OSError_SetRelAlarm_increment(), OSError_SetRelAlarm_cycle());
		break;
	case OSServiceId_SetAbsAlarm:
		syslog(LOG_INFO, "Error:%s=SetAbsAlarm(%d, %d, %d)", atk2_strerror(Error),
			   OSError_SetAbsAlarm_AlarmID(), OSError_SetAbsAlarm_start(), OSError_SetAbsAlarm_cycle());
		break;
	case OSServiceId_CancelAlarm:
		syslog(LOG_INFO, "Error:%s=CancelAlarm(%d)", atk2_strerror(Error), OSError_CancelAlarm_AlarmID());
		break;
	case OSServiceId_StartScheduleTableRel:
		syslog(LOG_INFO, "Error:%s=StartScheduleTableRel(%d, %d)", atk2_strerror(Error),
			   OSError_StartScheduleTableRel_ScheduleTableID(), OSError_StartScheduleTableRel_Offset());
		break;
	case OSServiceId_StartScheduleTableAbs:
		syslog(LOG_INFO, "Error:%s=StartScheduleTableAbs(%d, %d)", atk2_strerror(Error),
			   OSError_StartScheduleTableAbs_ScheduleTableID(), OSError_StartScheduleTableAbs_Start());
		break;
	case OSServiceId_StopScheduleTable:
		syslog(LOG_INFO, "Error:%s=StopScheduleTable(%d)", atk2_strerror(Error), OSError_StopScheduleTable_ScheduleTableID());
		break;
	case OSServiceId_NextScheduleTable:
		syslog(LOG_INFO, "Error:%s=NextScheduleTable(%d, %d)", atk2_strerror(Error),
			   OSError_NextScheduleTable_ScheduleTableID_From(), OSError_NextScheduleTable_ScheduleTableID_To());
		break;
	case OSServiceId_GetScheduleTableStatus:
		syslog(LOG_INFO, "Error:%s=GetScheduleTableStatus(%d, 0x%p)", atk2_strerror(Error),
			   OSError_GetScheduleTableStatus_ScheduleTableID(), OSError_GetScheduleTableStatus_ScheduleStatus());
		break;
	case OSServiceId_GetActiveApplicationMode:
		syslog(LOG_INFO, "Error:%s=GetActiveApplicationMode()", atk2_strerror(Error));
		break;
	case OSServiceId_StartCore:
		syslog(LOG_INFO, "Error:%s=StartCore(%d, %d)", atk2_strerror(Error), OSError_StartCore_CoreID(), OSError_StartCore_Status());
		break;
	case OSServiceId_StartNonAutosarCore:
		syslog(LOG_INFO, "Error:%s=StartNonAutosarCore(%d, %d)", atk2_strerror(Error), OSError_StartNonAutosarCore_CoreID(), OSError_StartNonAutosarCore_Status());
		break;
	case OSServiceId_StartOS:
		syslog(LOG_INFO, "Error:%s=StartOS(%d)", atk2_strerror(Error), OSError_StartOS_Mode());
		break;
	case OSServiceId_ShutdownAllCores:
		syslog(LOG_INFO, "Error:%s=ShutdownAllCores()", atk2_strerror(Error));
		break;
	case OSServiceId_IncrementCounter:
		syslog(LOG_INFO, "Error:%s=IncrementCounter(%d)", atk2_strerror(Error), OSError_IncrementCounter_CounterID());
		break;
	case OSServiceId_GetSpinlock:
		syslog(LOG_INFO, "Error:%s=GetSpinlock(%d)", atk2_strerror(Error), OSError_GetSpinlock_SpinlockId());
		break;
	case OSServiceId_ReleaseSpinlock:
		syslog(LOG_INFO, "Error:%s=ReleaseSpinlock(%d)", atk2_strerror(Error), OSError_ReleaseSpinlock_SpinlockId());
		break;
	case OSServiceId_TryToGetSpinlock:
		syslog(LOG_INFO, "Error:%s=TryToGetSpinlock(%d)", atk2_strerror(Error), OSError_GetSpinlock_SpinlockId());
		break;
	case OSServiceId_TaskMissingEnd:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_strerror(Error));
		break;
	case IOCServiceId_IOC_Send:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Write:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_SendGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_WriteGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Receive:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_Read:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_ReceiveGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_ReadGroup:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_IOC_EmptyQueue:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_send_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_write_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_receive_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	case IOCServiceId_ioc_read_generic:
		syslog(LOG_INFO, "Error:%s=MissingEnd()", atk2_ioc_strerror(Error));
		break;
	default:
		syslog(LOG_INFO, "Error:%s=UnKnownFunc()", atk2_strerror(Error));
		break;
	}

}   /* ErrorHook */
#endif /* CFG_USE_ERRORHOOK */


/*
 *  プレタスクフックルーチン
 *
 *  空ルーチンを呼出す
 */
#ifdef CFG_USE_PRETASKHOOK
void
PreTaskHook(void)
{
}   /* PreTaskHook */
#endif /* CFG_USE_PRETASKHOOK */


/*
 *  ポストタスクフックルーチン
 *
 *  空ルーチンを呼出す
 */
#ifdef CFG_USE_POSTTASKHOOK
void
PostTaskHook(void)
{
}   /* PostTaskHook */
#endif /* CFG_USE_POSTTASKHOOK */


/*
 *  スタートアップフックルーチン
 */
#ifdef CFG_USE_STARTUPHOOK
#ifdef TOPPERS_ENABLE_SYS_TIMER
extern void target_timer_initialize(void);
#endif /* TOPPERS_ENABLE_SYS_TIMER */

void
StartupHook(void)
{
	CoreIdType coreid = GetCoreID();

#ifdef TOPPERS_ENABLE_SYS_TIMER
	target_timer_initialize();
#endif /* TOPPERS_ENABLE_SYS_TIMER */
	if (coreid == OS_CORE_ID_MASTER) {
		syslog_initialize();
		syslog_msk_log(LOG_UPTO(LOG_INFO));
		InitSerial();
		print_banner();
	}
	else {
		InitSerial();
	}
	syslog(LOG_EMERG, "StartupHook @ core%d", coreid);
}   /* StartupHook */
#endif /* CFG_USE_STARTUPHOOK */


/*
 *  シャットダウンフックルーチン
 */
#ifdef CFG_USE_SHUTDOWNHOOK
#ifdef TOPPERS_ENABLE_SYS_TIMER
extern void target_timer_terminate(void);
#endif /* TOPPERS_ENABLE_SYS_TIMER */

void
ShutdownHook(StatusType Error)
{
	/* 終了ログ出力 */
	syslog(LOG_INFO, "");
	syslog(LOG_INFO, "Sample System ShutDown");
	syslog(LOG_INFO, "ShutDownCode:%s", atk2_strerror(Error));
	syslog(LOG_INFO, "");

	if (Error == E_OS_SYS_ASSERT_FATAL) {
		syslog(LOG_INFO, "fatal_file_name:%s", kernel_fatal_file_name);
		syslog(LOG_INFO, "fatal_line_num:%d", kernel_fatal_line_num);
	}

#ifdef TOPPERS_ENABLE_SYS_TIMER
	target_timer_terminate();
#endif /* TOPPERS_ENABLE_SYS_TIMER */
	TermSerial();

}   /* ShutdownHook */
#endif /* CFG_USE_SHUTDOWNHOOK */


/*
 *  プロテクションフックルーチン
 */
#ifdef CFG_USE_PROTECTIONHOOK
ProtectionReturnType
ProtectionHook(StatusType FatalError)
{
	StatusType ercd;

	syslog(LOG_INFO, "");
	syslog(LOG_INFO, "ProtectionHook");

	if (FatalError == E_OS_STACKFAULT) {
		syslog(LOG_INFO, "E_OS_STACKFAULT");
		ercd = PRO_SHUTDOWN;
	}
	else if (FatalError == E_OS_PROTECTION_EXCEPTION) {
		syslog(LOG_INFO, "E_OS_PROTECTION_EXCEPTION");
		ercd = PRO_IGNORE;
	}
	else {
		ercd = PRO_SHUTDOWN;
	}

	return(ercd);
}
#endif /* CFG_USE_PROTECTIONHOOK */


/*
 *  ICI ISRs
 */
ICISR(Core0_ICI_1) {

	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_1 run!");
	syslog(LOG_INFO, "Coer0_ICI_1 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_1 GetApplicationID() = %d", GetApplicationID());
	RaiseInterCoreInterrupt(Core1_ICI_2);
}

ICISR(Core1_ICI_1) {
	syslog(LOG_INFO, "/***********************/\nCoer1_ICI_1 run!");
	syslog(LOG_INFO, "Coer1_ICI_1 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer1_ICI_1 GetApplicationID() = %d", GetApplicationID());
	RaiseInterCoreInterrupt(Core0_ICI_2);
}

ICISR(Core0_ICI_2) {
	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_2 run!");
	syslog(LOG_INFO, "Coer0_ICI_2 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_2 GetApplicationID() = %d", GetApplicationID());
}

ICISR(Core1_ICI_2) {
	syslog(LOG_INFO, "/***********************/\nCoer1_ICI_2 run!");
	syslog(LOG_INFO, "Coer1_ICI_2 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer1_ICI_2 GetApplicationID() = %d", GetApplicationID());
}

ICISR(Core0_ICI_3) {
	syslog(LOG_INFO, "/***********************/\nCoer0_ICI_3 run!");
	syslog(LOG_INFO, "Coer0_ICI_3 GetISRID() = %d", GetISRID());
	syslog(LOG_INFO, "Coer0_ICI_3 GetApplicationID() = %d", GetApplicationID());
}


/*
 *  ログ表示用APIラッパ
 */
#ifdef MDCOM_DEBUG
static int  LOG_mdcom_init(mdcom_mask_t fifoch_mask, mdcom_time_t timeout)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_init(0x%04X, %d)", fifoch_mask, timeout);

    iRet = mdcom_init(fifoch_mask, timeout);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_sense_process(uint32_t *p_num_process)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_sense_process(0x%04X)", p_num_process);

    iRet = mdcom_sense_process(p_num_process);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, Number of process = %d",
        iRet, *p_num_process);

    return( iRet );
}

static int  LOG_mdcom_smemch_get(const mdcom_id_t smemch_id, void **buffer)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_smemch_get(%d, 0x%04X)", smemch_id, buffer);

    iRet = mdcom_smemch_get(smemch_id, buffer);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, Buffer address = 0x%04X",
        iRet, *buffer);

    return( iRet );
}

static int  LOG_mdcom_smemch_trylock(const mdcom_id_t smemch_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_smemch_trylock(%d)", smemch_id);

    iRet = mdcom_smemch_trylock(smemch_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_smemch_unlock(const mdcom_id_t smemch_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_smemch_unlock(%d)", smemch_id);

    iRet = mdcom_smemch_unlock(smemch_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_sense_fifoch(mdcom_mask_t *p_fifoch_mask)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_sense_fifoch(0x%04X)",
        p_fifoch_mask);

    iRet = mdcom_sense_fifoch(p_fifoch_mask);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, FIFO assignment = 0x%04X", iRet,
        *p_fifoch_mask);

    return( iRet );
}

static int  LOG_mdcom_fifoch_alloc(
            const mdcom_id_t fifoch_id, mdcom_id_t *block_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_alloc(%d, 0x%04X)",
        fifoch_id, block_id);

    iRet = mdcom_fifoch_alloc(fifoch_id, block_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, Block ID = %d", iRet,
        *block_id);

    return( iRet );
}

static int  LOG_mdcom_fifoch_free(
            const mdcom_id_t fifoch_id, const mdcom_id_t block_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_free(%d, %d)",
        fifoch_id, block_id);

    iRet = mdcom_fifoch_free(fifoch_id, block_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_fifoch_get(const mdcom_id_t fifoch_id,
            const mdcom_id_t block_id, void **buffer)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_get(%d, %d, 0x%04X)",
        fifoch_id, block_id, buffer);

    iRet = mdcom_fifoch_get(fifoch_id, block_id, buffer);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, Block address = 0x%04X",
        iRet, *buffer);

    return( iRet );
}

static int  LOG_mdcom_fifoch_enqueue(const mdcom_id_t fifoch_id,
            const mdcom_id_t block_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_enqueue(%d, %d)",
        fifoch_id, block_id);

    iRet = mdcom_fifoch_enqueue(fifoch_id, block_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_fifoch_dequeue(const mdcom_id_t fifoch_id,
            mdcom_id_t *block_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_dequeue(%d, 0x%04X)",
        fifoch_id, block_id);

    iRet = mdcom_fifoch_dequeue(fifoch_id, block_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d, Block ID = %d", iRet,
        *block_id);

    return( iRet );
}

static int  LOG_mdcom_fifoch_notify(const mdcom_id_t fifoch_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_notify(%d)",
        fifoch_id);

    iRet = mdcom_fifoch_notify(fifoch_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_fifoch_wait(
            const mdcom_id_t fifoch_id, const mdcom_time_t timeout)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_wait(%d, %d)",
        fifoch_id, timeout);

    iRet = mdcom_fifoch_wait(fifoch_id, timeout);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_filter_set(
            const mdcom_id_t fifoch_id, const mdcom_id_t  filter_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_filter_set(%d, %d)",
        fifoch_id, filter_id);

    iRet = mdcom_filter_set(fifoch_id, filter_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}

static int  LOG_mdcom_fifoch_init(const mdcom_id_t fifoch_id)
{
    int     iRet;

    syslog(LOG_NOTICE, "    Execute :  mdcom_fifoch_init(%d)",
        fifoch_id);

    iRet = mdcom_fifoch_init(fifoch_id);

    syslog(LOG_NOTICE, "    Complete : Return value = %d", iRet);

    return( iRet );
}
#endif	/* MDCOM_DEBUG */
