/*
 * MDCOM Multi Domain Communications Remote application
 *
 * $Id$
 */

#include <stdio.h>
#include <string.h>
#include <assert.h>
#include <unistd.h>
#include <stdlib.h>
#include <wait.h>
#include "mdcom.h"
#include "mdcom_config.h"
#include "tmout_test.h"

/*
 *	コンフィグレーション
 */


/*
 *  定数・型定義
 */
#define EXITCD_RESTART		0
#define EXITCD_EXIT			1


/*
 *  グローバル変数
 */
/* プロセスID */
int		pid_main;
int		pid_cmdproc;


/*
 *  関数プロトタイプ宣言
 */
/* External */

/* Internal */
static void		cmd_proccess(void);
static void		lock_testcmd(void);
static void		unlock_testcmd(void);


/*
 *  マクロ
 */


/*
 *  メインプロセス
 */
int main()
{
	int		pid;
	int		exit_status;
	uint8_t	exitcd;

    fflush(stdout);
    setbuf(stdout, NULL);

    printf("MDCOM timeout test program starts.\n");

    /* Get main process PID */
    pid_main = getpid();

	while( 1 )
	{
	    /*
         *  サブプロセスの生成
		 */
	    pid = fork();
	    if( pid == 0 )  {
	        cmd_proccess();
	    }
	    else if( pid < 0 )
	    {
	        printf("    Error : Failed create proccess.\n");
	        break;
	    }

		/*
	     *  メインプロセスの処理
	     */
		pid = wait(&exit_status);
		if( pid >= 0 )
		{
		    printf("    End cmd_proccess()\n");
			exitcd = exit_status >> 8;
			if( exitcd != EXITCD_RESTART )  {
				break;
			}
		}
		else  {
			break;
		}
	}

	return 0;
}


/*
 *  サブプロセス : コマンド処理
 */
static void
cmd_proccess(void)
{
	TESTCMD	*testcmd;
	volatile uint32_t	cmd;
	int		mdcom_retcd;

	pid_cmdproc = getpid();

    printf("    Start cmd_proccess()\n");

	mdcom_retcd = mdcom_init(0x0000001F, -1);
	assert(mdcom_retcd == MDCOM_SUCCESS);

	mdcom_retcd = mdcom_smemch_get(MDCOM_SHMEMCH_TESTCMD, (void **)&testcmd);
	assert(mdcom_retcd == MDCOM_SUCCESS);

	/* マスタアプリとの同期 */
	while( 1 )
	{
		cmd = testcmd->cmd;

		if( cmd == TESTCMD_READY )  {
			break;
		}
		else  {
			sleep(1);
		}
	}

	lock_testcmd();
	testcmd->cmd =  TESTCMD_NONE;
	testcmd->resp = TESTRESP_READY;
	unlock_testcmd();

	/* コマンド処理 */
	while( 1 )
	{
		/* コマンド受信待ち */
		do  {
			cmd = testcmd->cmd;
		}
		while( cmd == TESTCMD_NONE );

		/* コマンド処理 */
		switch( cmd )
		{
		/*
		 *  TESTCMD_REQ_APP_RESTART : MDCOMリモートアプリ再起動
         *     メインプロセスで本プロセスを再起動する。
		 */
		case TESTCMD_REQ_APP_RESTART:
		    printf("    cmd_proccess() : TESTCMD_REQ_APP_RESTART\n");
			lock_testcmd();
			testcmd->cmd = TESTCMD_NONE;
			unlock_testcmd();
			exit(EXITCD_RESTART);

		/*
		 *  TESTCMD_REQ_APP_EXIT :    MDCOMリモートアプリ終了
         *     本アプリを終了する。
		 */
		case TESTCMD_REQ_APP_EXIT:
		    printf("    cmd_proccess() : TESTCMD_REQ_APP_EXIT\n");
			lock_testcmd();
			testcmd->cmd = TESTCMD_NONE;
			unlock_testcmd();
			exit(EXITCD_EXIT);

		/*
		 *  TESTCMD_REQ_NOTIFY :     マスタへのNOTIFY要求
		 */
		case TESTCMD_REQ_NOTIFY:
		    printf("    cmd_proccess() : TESTCMD_REQ_NOTIFY\n");
			mdcom_retcd = mdcom_fifoch_notify(testcmd->param[0]);
			lock_testcmd();
			if( mdcom_retcd == MDCOM_SUCCESS )  {
				testcmd->resp = TESTRESP_SUCCESS;
			}
			else  {
				testcmd->resp = TESTRESP_MDCOM_FAILED;
			}
			testcmd->cmd = TESTCMD_NONE;
			unlock_testcmd();
			break;

		default:
			lock_testcmd();
			testcmd->cmd = TESTCMD_NONE;
			unlock_testcmd();
			break;
		}
	}
}


/*
 *  テスト制御用 shmemch ロック
 */
static void
lock_testcmd(void)
{
	int		mdcom_retcd;

	do  {
		mdcom_retcd = mdcom_smemch_trylock(MDCOM_SHMEMCH_TESTCMD);
	}
	while( mdcom_retcd == MDCOM_LOCKED );
	assert(mdcom_retcd == MDCOM_SUCCESS);
}


/*
 *  テスト制御用 shmemch アンロック
 */
static void
unlock_testcmd(void)
{
	int		mdcom_retcd;

	mdcom_retcd = mdcom_smemch_unlock(MDCOM_SHMEMCH_TESTCMD);
	assert(mdcom_retcd == MDCOM_SUCCESS);
}
