/*
 * MDCOM Multi Domain Communications Driver for Linux
 *
 * Copyright (C) Nagoya University
 * Daniel Sangorrin <daniel.sangorrin@gmail.com>
 * Shinya Honda     <honda@ertl.jp>  
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 *  @(#) $Id: mdcom_target_driver.h 953 2018-10-26 12:49:23Z ertl-honda $
 */

/*
 *  ARCHITECTURE
 */
#define __TARGET_ARCH_ARM 8

/*
 *  Configuration
 */
/* Use IPI driver */
#define USE_IPI_DRIVER
/* Use polling instead of IPI APU */
//#define USE_RPU_NOTIFY_POLLING

#ifdef USE_RPU_NOTIFY_POLLING
/* RPU polling time [ms] */
#define	RPU_POLL_TIME		100
#else	/* USE_RPU_NOTIFY_POLLING */
/* Clear interrupt event at IPI interrupt */
#define CLEAR_INT_EVENT
#endif	/* USE_RPU_NOTIFY_POLLING */


/*
 * SHARED MEMORY (allocated in kernel parameters)
 */
#define SREGION_PHYS_ADDRESS    0x7BF00000UL
#define SREGION_SIZE            0x00100000UL /* 1MB */
#define SREGION_WRITECOMBINE

#ifdef USE_RPU_NOTIFY_POLLING
extern void *mdcom_sregion;

#define MDCOM_SPINLOCK     (uint32_t *)(mdcom_sregion + SREGION_SIZE - (sizeof(uint32_t) * 2))
#define MDCOM_RPU_NOTIFY   (uint32_t *)(mdcom_sregion + SREGION_SIZE - sizeof(uint32_t))
#endif	/* USE_RPU_NOTIFY_POLLING */

/*
 * Errors
 */
#define MDCOM_SUCCESS         (0) /* function returned successfully */
#define MDCOM_ERR_SEMAPHORE   (1) /* semaphore handling error */


#ifdef  MDCOM_USE_LINUX_MUTEXDRIVER
#error "This option isn't supported. so ZynqMP ZCU102 doesn't have semahpore hardware."
#else  /* MDCOM_USE_LINUX_MUTEXDRIVER */

static int
_target_lock_init(void)
{
	
}

static int
_target_lock_deinit(void)
{
	
}

/*
 *  _target_ioctl_trylock()
 *
 *  Function : This function try to spin lock.
 *  Argument : None
 *  Return :   0 -     Success
 *             Other - Failed
 */
static int
_target_lock_trylock(void)
{
	/* Never call */
	return 0;
}

/*
 *  _target_ioctl_tryunlock()
 *
 *  Function : This function try to spin unlock.
 *  Argument : None
 *  Return :   0 -     Success
 *             Other - Failed
 */
static int
_target_lock_unlock(void)
{
	/* Never call */
	return 0;
}
#endif  /* MDCOM_USE_LINUX_MUTEXDRIVER */



/*
 * Prototypes in mdcom-ipi.c
 */
extern void mdcom_ipi_generate_int(void);
extern void mdcom_ipi_clear_int(void);

/*
 *  send_r2m_irq()
 *
 *  Function : This function occurs the interrupt to master core.
 *  Argument : None
 *  Return :   None
 */
static inline void
_target_int_send(void)
{
	mdcom_ipi_generate_int();
}

static void
_target_int_init(void)
{
	/* do nothing */
}

static void
_target_int_exit(void)
{
	/* do nothing */
}

/*
 *  __mdcom_kernel_clear_int()
 *
 *  Function : This function clears the ipi-interrupt event.
 *  Argument : None
 *  Return :   None
 */
static void
_target_int_clear(void)
{
	/* Clear Interrupt */
	mdcom_ipi_clear_int();
}
