/*
 *  TOPPERS/MDCOM
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Multi Domain Communication Module
 *
 *  Copyright (C) 2016-2019 by Embedded and Real-Time Systems Laboratory
 *     Graduate School of Information Science, Nagoya University, JAPAN
 *
 *  上記著作権者は，以下の(1)～(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 *  @(#) $Id: mdcom_target.h 1006 2019-01-31 14:34:27Z ertl-honda $
 */
#include "Os.h"
#include "core_sil.h"

#define MDCOM_TYPE_API       MDCOM_TYPE_API_ATK2
#define MDCOM_TYPE_OS        MDCOM_TYPE_OS_SC1_MC
#define MDCOM_TYPE_EMV       MDCOM_TYPE_EMV_AMP
#define MDCOM_TYPE_ROLL      MDCOM_TYPE_ROLL_MASTER

#define MDCOM_DOMAIN_NAME    "ATK2(CR5)"

#define MDCOM_TARGET_CR5
#include "../mdcom_target_common.h"

#define MDCOM_INT_PRIORITY   2
#define MDCOM_INTNO          (0xFFFF0000|XIPI_INTNO)
#define MDCOM_INT_STACK_SIZE 1024

#define SREGION_PHYS_ADDRESS MDCOM_SHMEM_ADDRESS

static inline void
__mdcom_dcache_flush(void)
{
    
}

#if MDCOM_TYPE_API == MDCOM_TYPE_API_ATK2

static inline bool_t
__mdcom_kernel_mutex_trylock(uint32_t *p_lock)
{
    int locked;
    bool_t ret = false;
    SIL_PRE_LOC;

    SIL_LOC_SPN();

    Asm("   mov      r2, #0x01   \n"
        "\t ldrex    r1, [%1]    \n"
        "\t cmp      r1, #0x00   \n"
        "\t strexeq  r1, r2,[%1] \n"
        "\t mov      %0, r1      \n"
        :"=r"(locked):"r"(p_lock):"r1","r2", "cc");

    if(locked == 0){
        /* ロック取得成功 */
        SIL_DMB;
        Asm("":::"memory");
        ret = true;
    }
   
    SIL_UNL_SPN();

    return ret;    
}


static inline void
__mdcom_kernel_mutex_unlock(uint32_t *p_lock)
{
    *p_lock = 0;

    Asm("":::"memory");
    SIL_DMB;
    *p_lock = 0;
    SIL_DMB;
}

#endif /* MDCOM_TYPE_API  MDCOM_TYPE_API_ATK2 */
