/*
 *  TOPPERS/MDCOM
 *      Toyohashi Open Platform for Embedded Real-Time Systems/
 *      Multi Domain Communication Module
 *
 *  Copyright (C) 2016 by Embedded and Real-Time Systems Laboratory
 *     Graduate School of Information Science, Nagoya University, JAPAN
 *
 *  上記著作権者は，以下の(1)〜(4)の条件を満たす場合に限り，本ソフトウェ
 *  ア（本ソフトウェアを改変したものを含む．以下同じ）を使用・複製・改
 *  変・再配布（以下，利用と呼ぶ）することを無償で許諾する．
 *  (1) 本ソフトウェアをソースコードの形で利用する場合には，上記の著作
 *      権表示，この利用条件および下記の無保証規定が，そのままの形でソー
 *      スコード中に含まれていること．
 *  (2) 本ソフトウェアを，ライブラリ形式など，他のソフトウェア開発に使
 *      用できる形で再配布する場合には，再配布に伴うドキュメント（利用
 *      者マニュアルなど）に，上記の著作権表示，この利用条件および下記
 *      の無保証規定を掲載すること．
 *  (3) 本ソフトウェアを，機器に組み込むなど，他のソフトウェア開発に使
 *      用できない形で再配布する場合には，次のいずれかの条件を満たすこ
 *      と．
 *    (a) 再配布に伴うドキュメント（利用者マニュアルなど）に，上記の著
 *        作権表示，この利用条件および下記の無保証規定を掲載すること．
 *    (b) 再配布の形態を，別に定める方法によって，TOPPERSプロジェクトに
 *        報告すること．
 *  (4) 本ソフトウェアの利用により直接的または間接的に生じるいかなる損
 *      害からも，上記著作権者およびTOPPERSプロジェクトを免責すること．
 *      また，本ソフトウェアのユーザまたはエンドユーザからのいかなる理
 *      由に基づく請求からも，上記著作権者およびTOPPERSプロジェクトを
 *      免責すること．
 *
 *  本ソフトウェアは，無保証で提供されているものである．上記著作権者お
 *  よびTOPPERSプロジェクトは，本ソフトウェアに関して，特定の使用目的
 *  に対する適合性も含めて，いかなる保証も行わない．また，本ソフトウェ
 *  アの利用により直接的または間接的に生じたいかなる損害に関しても，そ
 *  の責任を負わない．
 *
 * [target_common.h]
 *
 * This file contains target-dependent constant definitions used both
 * monitor and user program.
 *
 * @(#) $Id: mdcom_target_common.h 1037 2019-05-30 01:55:30Z ertl-honda $  
 */
#ifndef _MDCOM_TARGET_COMMON_H_
#define _MDCOM_TARGET_COMMON_H_

/*
 *  Configuration
 */
/* Use polling instead of IPI APU */
//#define USE_RPU_NOTIFY_POLLING

/*
 *  Use shard memory as Shared Region on MDCOM
 */
#define MDCOM_SHMEM_ADDRESS  0x7BF00000  /* DDR */
#define MDCOM_SHMEM_SIZE     0x00100000  /* 1MB */
#define MDCOM_SHMEM_SIZE_RSAE RSAE_1M

#ifdef USE_RPU_NOTIFY_POLLING
#define MDCOM_SPINLOCK          (uint32_t *)(MDCOM_SHMEM_ADDRESS + MDCOM_SHMEM_SIZE - (sizeof(uint32_t) * 2))
#define MDCOM_RPU_NOTIFY        (uint32_t *)(MDCOM_SHMEM_ADDRESS + MDCOM_SHMEM_SIZE - sizeof(uint32_t))
#endif  /* USE_RPU_NOTIFY_POLLING */

/*
 *  Register Defintion
 */
#define XIPI_INTNO_RPU_0 (33+32)
#define XIPI_INTNO_RPU_1 (34+32)
#define XIPI_INTNO_APU   (35+32)
#define XIPI_INTNO_PL0   (29+32)

#define XIPI_TARGET_BIT_APU    0
#define XIPI_TARGET_BIT_RPU_0  8
#define XIPI_TARGET_BIT_RPU_1  9
#define XIPI_TARGET_BIT_PL0   24

#define XIPI_ALL_MASK          0x0F0F0301U
#define XIPI_TARGET_MASK_APU   (1<<XIPI_TARGET_BIT_APU)
#define XIPI_TARGET_MASK_RPU_0 (1<<XIPI_TARGET_BIT_RPU_0)
#define XIPI_TARGET_MASK_RPU_1 (1<<XIPI_TARGET_BIT_RPU_1)
#define XIPI_TARGET_MASK_PL0   (1<<XIPI_TARGET_BIT_PL0)

#define XIPI_BASE           0xFF300000

#define XIPI_APU_TRIG       (XIPI_BASE + 0x00000)
#define XIPI_APU_OBS        (XIPI_BASE + 0x00004)
#define XIPI_APU_ISR        (XIPI_BASE + 0x00010)
#define XIPI_APU_IER        (XIPI_BASE + 0x00018)
#define XIPI_APU_IDR        (XIPI_BASE + 0x0001C)

#define XIPI_PL0_TRIG       (XIPI_BASE + 0x40000)
#define XIPI_PL0_OBS        (XIPI_BASE + 0x40004)
#define XIPI_PL0_ISR        (XIPI_BASE + 0x40010)
#define XIPI_PL0_IER        (XIPI_BASE + 0x40018)
#define XIPI_PL0_IDR        (XIPI_BASE + 0x4001C)

#define XIPI_RPU_0_TRIG     (XIPI_BASE + 0x10000)
#define XIPI_RPU_0_OBS      (XIPI_BASE + 0x10004)
#define XIPI_RPU_0_ISR      (XIPI_BASE + 0x10010)
#define XIPI_RPU_0_IER      (XIPI_BASE + 0x10018)
#define XIPI_RPU_0_IDR      (XIPI_BASE + 0x1001C)

#define XIPI_RPU_1_TRIG     (XIPI_BASE + 0x20000)
#define XIPI_RPU_1_OBS      (XIPI_BASE + 0x20004)
#define XIPI_RPU_1_ISR      (XIPI_BASE + 0x20010)
#define XIPI_RPU_1_IER      (XIPI_BASE + 0x20018)
#define XIPI_RPU_1_IDR      (XIPI_BASE + 0x2001C)

#ifdef MDCOM_TARGET_CA53

#define XIPI_TARGET_MASK     XIPI_TARGET_MASK_RPU_0

#define XIPI_TRIG    XIPI_PL0_TRIG
#define XIPI_OBS     XIPI_PL0_OBS
#define XIPI_ISR     XIPI_PL0_ISR
#define XIPI_IER     XIPI_PL0_IER
#define XIPI_IDR     XIPI_PL0_IDR

#define XIPI_INTNO XIPI_INTNO_PL0

#elif defined(MDCOM_TARGET_CR5)

#define XIPI_TARGET_MASK     XIPI_TARGET_MASK_PL0

#define XIPI_TRIG    XIPI_RPU_0_TRIG
#define XIPI_OBS     XIPI_RPU_0_OBS
#define XIPI_ISR     XIPI_RPU_0_ISR
#define XIPI_IER     XIPI_RPU_0_IER
#define XIPI_IDR     XIPI_RPU_0_IDR

#define XIPI_INTNO XIPI_INTNO_RPU_0

#else
#error CA53 or CR5
#endif /* MDCOM_TARGET_CA53 */

#if (MDCOM_TYPE_API == MDCOM_TYPE_API_TOPPERS) || (MDCOM_TYPE_API == MDCOM_TYPE_API_ATK2)
/*
 *  ITRON系TOPPERSカーネル
 *  AUTOSAR系TOPPERSカーネル
 */
#define SEND_IRQ_OK 1
#define SEND_IRQ_NG 0

#define ELOCKED     -6
#define EUNLOCKED   -7

static inline void
__mdcom_send_irq(void)
{
#ifdef  USE_RPU_NOTIFY_POLLING
	*MDCOM_RPU_NOTIFY = true;
#else   /* USE_RPU_NOTIFY_POLLING */
	volatile uint32_t   reg32;

	/*
	 * 割込み要求のペンディングはチェックしない．
	 *  リモート : 上位レイヤではマスタからのACKを受け取ってから次に進むため．
	 *  マスタ   : 割込みハンドラからの呼び出しではリモート側はACK待ちになっている．
	 *             notifyではチャネル毎のデータ構造にnotify要求を書き込んでいるため．
	 */
	sil_wrw_mem((uint32_t *)XIPI_TRIG, XIPI_TARGET_MASK);
#endif  /* USE_RPU_NOTIFY_POLLING */
}

static inline void
__mdcom_kernel_init_init()
{
	/* Disable Interrupt */
	sil_wrw_mem((uint32_t *)XIPI_IDR, XIPI_ALL_MASK);
	/* Clear Interrupt */
	sil_wrw_mem((uint32_t *)XIPI_ISR, XIPI_ALL_MASK);
	/* Enable Interrupt */
	sil_wrw_mem((uint32_t *)XIPI_IER, XIPI_TARGET_MASK);
}

static inline void
__mdcom_kernel_clear_int(void)
{
	/* Clear Interrupt */
	sil_wrw_mem((uint32_t *)XIPI_ISR, XIPI_TARGET_MASK);
}

static inline void
__mdcom_kernel_mutex_init(void)
{
}
#endif /* MDCOM_TYPE_API  MDCOM_TYPE_API_TOPPERS || MDCOM_TYPE_API_ATK2 */

#endif /* _MDCOM_TARGET_COMMON_H_ */
